import {Button, Group, Modal, ModalProps, Text} from "@mantine/core";
import {useDisclosure} from "@mantine/hooks";
import {IconAlertTriangle} from "@tabler/icons-react";
import {useState} from "react";


/**
 * This hook provides a confirmation modal that can be used to confirm actions.
 * It returns a function to toggle the modal and the modal component itself.
 * @param title - the title of the modal (optional)
 * @param description - the description of the modal
 * @param onConfirm - the function to call when the user confirms
 * @param onCancel - the function to call when the user cancels (optional)
 * @param props - additional props for the modal
 */
export const useConfirmModal = <T, >(
    {title, description, onConfirm, onCancel, ...props}: {
        title?: string,
        description: string,
        onConfirm: (t?: T) => void,
        onCancel?: () => void
    } & Omit<ModalProps, "opened" | "onClose">
) => {

    const [showConfirmModal, handler] = useDisclosure(false)

    const [data, setData] = useState<T | undefined>(undefined)

    const ConfirmModal = () => {
        return <>
            <Modal
                size={"sm"}
                {...props}
                opened={showConfirmModal}
                onClose={handler.close}
                withCloseButton={false}
                title={title ?? "Bestätigen"}
            >
                <div className={"stack"}>
                    <Text size="sm">
                        {description}
                    </Text>
                    <Group justify={"start"}>
                        <Button
                            variant={"light"}
                            color={"green"}
                            size={"xs"}
                            onClick={() => {
                                handler.close()
                                onCancel?.()
                                setData(undefined)
                            }}>
                            Abbrechen
                        </Button>
                        <Button
                            onClick={() => {
                                handler.close()
                                onConfirm(data)
                                setData(undefined)
                            }}
                            size={"xs"}
                            leftSection={<IconAlertTriangle/>}
                            color={"orange"}
                        >
                            Weiter
                        </Button>
                    </Group>
                </div>
            </Modal>
        </>
    }

    return {
        showConfirmModal,
        toggleConfirmModal: (t?: T) => {
            handler.toggle()
            setData(t)
        },
        ConfirmModal
    }
}