import {useChangeEmail} from "@/components/auth/modals/hooks.ts";
import {isEmail, useForm} from "@mantine/form";
import {Button, Center, Group, Modal, PasswordInput, TextInput} from "@mantine/core";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {IconArrowForward, IconAt, IconKey, IconX} from "@tabler/icons-react";
import {useMutation} from "@tanstack/react-query";
import {showSuccessNotification} from "@/components/util.tsx";
import {useSearchParams} from "react-router-dom";

import EmailSVG from "@/illustrations/email.svg?react"
import PromptLoginModal from "@/components/auth/modals/PromptLoginModal.tsx";

export const CHANGE_EMAIL_TOKEN_KEY = "changeEmailToken"


const RequestEmailChangeModal = ({open, onClose}: {
    open: boolean,
    onClose: () => void
}) => {
    const {pb} = usePB()

    const formValues = useForm({
        initialValues: {
            email: "",
        },
        validate: {
            email: isEmail("Ungültige E-Mail Adresse"),
        }
    })

    const mutation = useMutation({
        mutationFn: async () => {
            await pb.collection("users").requestEmailChange(formValues.values.email)
        },
        onSuccess: () => {
            formValues.reset()
            showSuccessNotification("Wir haben dir eine E-Mail an deine neue Adresse geschickt, bitte bestätige diese")
            onClose()
        }
    })

    return <Modal opened={open} onClose={onClose}
                  title={"E-Mail ändern"}>

        <form className={"stack"} onSubmit={formValues.onSubmit(() => mutation.mutate())}>

            <Center>
                <EmailSVG height={"200px"} width={"200px"}/>
            </Center>

            <PocketBaseErrorAlert error={mutation.error}/>

            <TextInput
                label={"Neue E-Mail Adresse"}
                description={"Bitte gebe die neue E-Mail Adresse sein"}
                placeholder={"E-Mail Adresse"}
                leftSection={<IconAt/>}
                required
                {...formValues.getInputProps("email")}
            />

            <Group>
                <Button
                    leftSection={<IconX/>}
                    onClick={onClose}
                    variant={"light"}
                >
                    Abbrechen
                </Button>

                <Button
                    type={"submit"}
                    leftSection={<IconArrowForward/>}
                    loading={mutation.isPending}
                >
                    E-Mail ändern
                </Button>
            </Group>
        </form>
    </Modal>
}

const ConfirmEmailChangeModal = ({open, onClose, token}: {
    open: boolean,
    onClose: () => void,
    token: string
}) => {
    const {pb, refreshUser} = usePB()

    const formValues = useForm({
        initialValues: {
            password: "",
        }
    })

    const mutation = useMutation({
        mutationFn: async () => {
            await pb.collection("users").confirmEmailChange(
                token,
                formValues.values.password,
            )
        },
        onSuccess: () => {
            formValues.reset()
            showSuccessNotification("E-Mail erfolgreich geändert")
            refreshUser()
            onClose()
        }
    })

    return <Modal opened={open} onClose={onClose}
                  title={"E-Mail ändern"}>

        <form className={"stack"} onSubmit={formValues.onSubmit(() => mutation.mutate())}>

            <Center>
                <EmailSVG height={"200px"} width={"200px"}/>
            </Center>

            <PocketBaseErrorAlert error={mutation.error}/>

            <PasswordInput
                label={"Passwort"}
                description={"Bitte gebe dein Passwort ein um deine E-Mail zu ändern"}
                placeholder={"Passwort"}
                leftSection={<IconKey/>}
                required
                {...formValues.getInputProps("password")}
            />

            <Group>
                <Button
                    leftSection={<IconX/>}
                    onClick={onClose}
                    variant={"light"}
                >
                    Abbrechen
                </Button>

                <Button
                    type={"submit"}
                    leftSection={<IconArrowForward/>}
                    loading={mutation.isPending}
                >
                    E-Email ändern
                </Button>
            </Group>
        </form>
    </Modal>
}

export default function ChangeEmailModal() {
    const {value, handler} = useChangeEmail()

    const {user} = usePB()

    const [searchParams] = useSearchParams()

    const token = searchParams.get(CHANGE_EMAIL_TOKEN_KEY)

    if (value && !user) {
        return <PromptLoginModal
            onAbort={handler.close}
            description={"Du musst eingeloggt sein um deine E-Mail Adresse zu ändern"}
        />
    }

    return <>
        {token ? (
            <ConfirmEmailChangeModal open={value} onClose={handler.close} token={token}/>
        ) : (
            <RequestEmailChangeModal open={value} onClose={handler.close}/>
        )}
    </>
}