import {useSearchParams} from "react-router-dom";
import {useEffect, useState} from "react";
import {useMutation} from "@tanstack/react-query";
import {usePB} from "@/lib/pocketbase.tsx";
import {showErrorNotification, showSuccessNotification} from "@/components/util.tsx";
import {Alert, Button, Group, Modal, TextInput, Title} from "@mantine/core";
import {IconAt} from "@tabler/icons-react";
import {useLogin} from "@/components/auth/modals/hooks.ts";

export const EMAIL_TOKEN_KEY = "emailVerificationToken"

/**
 * This Component is used to verify auth tokens.
 *
 * The following tokens are supported:
 *
 * - Email verification
 * - Password reset
 * - Email change
 *
 * The token is extracted from the URL and the verification is done.
 */
export default function EmailTokenVerification() {

    const [searchParams] = useSearchParams()

    const {handler: loginHandler} = useLogin()

    const [email, setEmail] = useState<string>("")

    const {pb, user, refreshUser} = usePB()

    const verifyTokenMutation = useMutation({
        mutationFn: async (token: string) => {
            await pb.collection("users").confirmVerification(token)
        },
        onSuccess: () => {
            showSuccessNotification("E-Mail erfolgreich verifiziert")
            refreshUser()
            if (!user) {
                loginHandler.open()
            }
        },
        onError: () => {
            showErrorNotification("Fehler beim Verifizieren der E-Mail")
        }
    })

    useEffect(() => {
        const token = searchParams.get(EMAIL_TOKEN_KEY)
        if (token !== null) {
            verifyTokenMutation.mutate(token)
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [searchParams])

    return <Modal opened={user ? !user.verified : false} onClose={() => undefined} withCloseButton={false}>
        <div className={"stack"}>
            <Title ta={"center"} order={3} c={"orange"}>Token Verifizierung</Title>
            <Alert>
                Deine E-Mail muss verifiziert werden.
                Bitte schau in dein E-Mail Postfach und klicke auf den Button in der E-Mail.
            </Alert>
            <TextInput
                type={"email"}
                label={"E-Mail Adresse"}
                placeholder={"E-Mail"}
                description={"Bitte gib deine E-Mail Adresse ein um den Token zu erhalten"}
                leftSection={<IconAt/>}
                value={email}
                onChange={(e) => setEmail(e.currentTarget.value)}
            />
            <Group>
                <Button
                    disabled={email.length === 0}
                    onClick={() => {
                        pb.collection("users").requestVerification(email)
                    }}
                >
                    Token erneut senden
                </Button>
            </Group>
        </div>
    </Modal>
}