import {useForgotPassword, useLogin} from "@/components/auth/modals/hooks.ts";
import {isEmail, useForm} from "@mantine/form";
import {Button, Center, Collapse, Group, Modal, PasswordInput, TextInput} from "@mantine/core";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {IconAt, IconKey, IconPassword, IconX} from "@tabler/icons-react";
import {useMutation} from "@tanstack/react-query";
import {showSuccessNotification} from "@/components/util.tsx";
import {useSearchParams} from "react-router-dom";

import PasswordSVG from "@/illustrations/boy-with-key.svg?react";
import PasswordStrengthMeter, {getPasswordStrength} from "@/components/input/PasswordStrengthMeter.tsx";

export const PWD_RESET_TOKEN_KEY = "passwordResetToken"


const RequestResetPasswordModal = ({open, onClose}: {
    open: boolean,
    onClose: () => void
}) => {
    const {pb} = usePB()

    const formValues = useForm({
        initialValues: {
            email: "",
        },
        validate: {
            email: isEmail("Ungültige E-Mail Adresse"),
        }
    })

    const requestResetPasswordMutation = useMutation({
        mutationFn: async () => {
            await pb.collection("users").requestPasswordReset(formValues.values.email)
        },
        onSuccess: () => {
            formValues.reset()
            showSuccessNotification("Passwort zurücksetzen erfolgreich, bitte überprüfe deine E-Mails")
        }
    })

    return <Modal opened={open} onClose={onClose}
                  title={"Passwort zurücksetzen"}>

        <form className={"stack"} onSubmit={formValues.onSubmit(() => requestResetPasswordMutation.mutate())}>

            <Center>
                <PasswordSVG height={"200px"} width={"200px"}/>
            </Center>

            <PocketBaseErrorAlert error={requestResetPasswordMutation.error}/>

            <TextInput
                label={"E-Mail Adresse"}
                description={"Bitte gebe deine E-Mail Adresse ein"}
                placeholder={"E-Mail Adresse"}
                leftSection={<IconAt/>}
                required
                {...formValues.getInputProps("email")}
            />

            <Group>
                <Button
                    leftSection={<IconX/>}
                    onClick={onClose}
                    variant={"light"}
                >
                    Abbrechen
                </Button>

                <Button
                    type={"submit"}
                    leftSection={<IconPassword/>}
                    loading={requestResetPasswordMutation.isPending}
                >
                    Zurücksetzen
                </Button>
            </Group>
        </form>
    </Modal>
}

const ResetPasswordModal = ({open, onClose, token}: {
    open: boolean,
    onClose: () => void,
    token: string
}) => {
    const {pb, user} = usePB()

    const {handler: loginHandler} = useLogin()

    const formValues = useForm({
        initialValues: {
            password: "",
            passwordConfirm: "",
        },
        validate: {
            password: (val) => getPasswordStrength(val) !== 100 ? "Das Passwort ist zu schwach" : null,
            passwordConfirm: (val, values) => val !== values.password ? "Die Passwörter stimmen nicht überein" : null

        }
    })

    const requestResetPasswordMutation = useMutation({
        mutationFn: async () => {
            await pb.collection("users").confirmPasswordReset(
                token,
                formValues.values.password,
                formValues.values.passwordConfirm
            )
        },
        onSuccess: () => {
            formValues.reset()
            showSuccessNotification("Passwort erfolgreich zurückgesetzt")
            if (!user) {
                loginHandler.open()
            }
        }
    })

    return <Modal opened={open} onClose={onClose}
                  title={"Passwort zurücksetzen"}>

        <form className={"stack"} onSubmit={formValues.onSubmit(() => requestResetPasswordMutation.mutate())}>

            <Center>
                <PasswordSVG height={"200px"} width={"200px"}/>
            </Center>

            <PocketBaseErrorAlert error={requestResetPasswordMutation.error}/>

            <PasswordInput
                label={"Neues Passwort"}
                description={"Das Passwort muss mindestens 8 Zeichen lang sein"}
                placeholder={"Neues Passwort"}
                leftSection={<IconKey/>}
                required
                {...formValues.getInputProps("password")}
            />

            <Collapse in={formValues.values.password.length > 0} className={"stack"}>
                <PasswordInput
                    label={"Neues Passwort wiederholen"}
                    description={"Bitte wiederhole dein Passwort"}
                    placeholder={"Neues Passwort wiederholen"}
                    leftSection={<IconKey/>}
                    required
                    {...formValues.getInputProps("passwordConfirm")}
                />
                <PasswordStrengthMeter
                    password={formValues.values.password}
                    passwordConfirm={formValues.values.passwordConfirm}
                />
            </Collapse>

            <Group>
                <Button
                    leftSection={<IconX/>}
                    onClick={onClose}
                    variant={"light"}
                >
                    Abbrechen
                </Button>

                <Button
                    type={"submit"}
                    leftSection={<IconPassword/>}
                    loading={requestResetPasswordMutation.isPending}
                >
                    Zurücksetzen
                </Button>
            </Group>
        </form>
    </Modal>
}

export default function ForgotPasswordModal() {
    const {value, handler} = useForgotPassword()

    const [searchParams] = useSearchParams()

    const token = searchParams.get(PWD_RESET_TOKEN_KEY)

    return <>
        {token ? (
            <ResetPasswordModal open={value} onClose={handler.close} token={token}/>
        ) : (
            <RequestResetPasswordModal open={value} onClose={handler.close}/>
        )}
    </>
}