import {Link} from "react-router-dom";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {useForm} from "@mantine/form";
import {useMutation} from "@tanstack/react-query";
import {
    Anchor,
    Button,
    Center,
    Checkbox,
    Divider,
    Group,
    Modal,
    PasswordInput,
    Radio,
    Text,
    TextInput,
    Title
} from "@mantine/core";
import LoginSVG from "@/illustrations/boy-with-key.svg?react"
import {useForgotPassword, useLogin, useRegister} from "@/components/auth/modals/hooks.ts";
import {showSuccessNotification} from "@/components/util.tsx";


export default function LoginModal() {

    const {value, handler} = useLogin()

    const {ldapLogin, guestLogin, user} = usePB()

    const {handler: registerHandler} = useRegister()

    const {handler: forgorPasswordHandler} = useForgotPassword()

    const formValues = useForm({
        initialValues: {
            username: "",
            password: "",
            authMethod: "ldap" as "ldap" | "guest",
            terms: false
        }
    })

    const loginMutation = useMutation({
        mutationFn: async () => {
            if (formValues.values.authMethod === "guest") {
                await guestLogin(formValues.values.username, formValues.values.password)
            } else {
                await ldapLogin(formValues.values.username, formValues.values.password)
            }
        },
        onSuccess: () => {
            handler.close()
            formValues.reset()
            showSuccessNotification("Erfolgreich eingeloggt")
        }
    })

    return <>
        <Modal opened={value && !user} onClose={handler.close} withCloseButton={false} size={"sm"}>
            <form className={"stack"} onSubmit={formValues.onSubmit(() => loginMutation.mutate())}>
                <Title ta={"center"} order={3}>StuVe IT Login</Title>

                <Center>
                    <LoginSVG height={"200px"} width={"200px"}/>
                </Center>

                <Radio.Group
                    {...formValues.getInputProps("authMethod")}
                    name="authMethod"
                    withAsterisk
                >
                    <Group justify={"space-around"}>
                        <Radio value="ldap" label="StuVe IT Account"/>
                        <Radio value="guest" label="Gäste Account"/>
                    </Group>
                </Radio.Group>

                <TextInput
                    label={"Anmeldename"}
                    placeholder={
                        formValues.values.authMethod === "ldap" ?
                            "vorname.nachname" : "Anmeldename"
                    }
                    {...formValues.getInputProps("username")}
                />

                <PasswordInput
                    label={"Passwort"}
                    placeholder={"Passwort"}
                    {...formValues.getInputProps("password")}
                />

                <Checkbox
                    required
                    label={
                        <Text>
                            Ich habe die <Anchor
                            component={Link}
                            target={"_blank"}
                            to={"/legal/terms-and-conditions"}
                        >
                            AGB der StuVe
                        </Anchor> gelesen und nehme sie zur Kenntnis
                        </Text>
                    }
                    {...formValues.getInputProps("terms", {type: "checkbox"})}
                />

                <PocketBaseErrorAlert error={loginMutation.error}/>

                <Button
                    loading={loginMutation.isPending}
                    disabled={formValues.values.username === "" || formValues.values.password === "" || !formValues.values.terms}
                    type={"submit"}
                >
                    Einloggen
                </Button>

                <Divider label={"oder"}/>

                <Group justify={"space-evenly"}>
                    <Button
                        size={"compact-xs"}
                        variant={"transparent"}
                        onClick={() => {
                            handler.close()
                            forgorPasswordHandler.open()
                        }}
                    >
                        Passwort vergessen?
                    </Button>

                    <Button
                        size={"compact-xs"}
                        variant={"transparent"}
                        onClick={() => {
                            handler.close()
                            registerHandler.open()
                        }}
                    >
                        Gast Account anlegen
                    </Button>
                </Group>
            </form>
        </Modal>
    </>
}