import {useRegister} from "@/components/auth/modals/hooks.ts";
import {isEmail, useForm} from "@mantine/form";
import {Alert, Anchor, Button, Checkbox, Collapse, Group, Modal, PasswordInput, Text, TextInput} from "@mantine/core";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {IconAt, IconKey, IconUser, IconUserPlus, IconX} from "@tabler/icons-react";
import PasswordStrengthMeter, {getPasswordStrength} from "@/components/input/PasswordStrengthMeter.tsx";
import {useMutation} from "@tanstack/react-query";
import {showSuccessNotification} from "@/components/util.tsx";
import {Link} from "react-router-dom";

export default function RegisterModal() {
    const {value, handler} = useRegister()

    const {user, pb} = usePB()

    const formValues = useForm({
        initialValues: {
            username: "",
            email: "",
            password: "",
            passwordConfirm: "",
            sn: "",
            givenName: "",
            terms: false,
            privacy: false
        },
        validate: {
            email: isEmail("Ungültige E-Mail Adresse"),
            username: (val) => val.length < 3 ? "Der Anmeldename muss mindestens 3 Zeichen lang sein" : null,
            password: (val) => getPasswordStrength(val) !== 100 ? "Das Passwort ist zu schwach" : null,
            passwordConfirm: (val, values) => val !== values.password ? "Die Passwörter stimmen nicht überein" : null,
            terms: (val) => !val ? "Du musst die AGB akzeptieren" : null,
            privacy: (val) => !val ? "Du musst die Datenschutzerklärung akzeptieren" : null
        }
    })

    const registerMutation = useMutation({
        mutationFn: async () => {
            await pb.collection("users").create({
                ...formValues.values,
                REALM: "GUEST"
            })
            await pb.collection("users").requestVerification(formValues.values.email)
        },
        onSuccess: () => {
            handler.close()
            formValues.reset()
            showSuccessNotification(
                "Account erfolgreich angelegt. " +
                "Bitte überprüfe deine E-Mails und klicke auf den Bestätigungslink. " +
                "Das verschicken der E-Mail kann einige Minuten dauern. " +
                "Falls du keine E-Mail erhalten hast überprüfe bitte deinen Spam Ordner."
            )
        }
    })

    return <>
        <Modal size={"lg"} opened={value && !user} onClose={handler.close}
               title={"Gast Account anlegen"}>

            <form className={"stack"} onSubmit={formValues.onSubmit(() => registerMutation.mutate())}>
                <Alert>
                    Hier kannst du einen Gast Account anlegen. Mit diesem Account kannst du dich zum Beispiel für
                    ein Event anmelden.
                    <br/>
                    Falls du einen StuVe IT Account hast, kannst du auch diesen Nutzen um dich anzumelden.
                </Alert>

                <PocketBaseErrorAlert error={registerMutation.error}/>

                <TextInput
                    label={"E-Mail Adresse"}
                    description={"Diese E-Mail Adresse ist für niemanden sichtbar"}
                    placeholder={"E-Mail Adresse"}
                    leftSection={<IconAt/>}
                    required
                    {...formValues.getInputProps("email")}
                />

                <Collapse in={formValues.values.email.length > 0} className={"stack"}>

                    <TextInput
                        label={"Anmeldename"}
                        description={"Dein Anmeldename ist für eingeloggte Personen sichtbar"}
                        placeholder={"Anmeldename"}
                        leftSection={<IconUser/>}
                        required
                        {...formValues.getInputProps("username")}
                    />

                    <Collapse in={formValues.values.username.length > 0} className={"stack"}>

                        <PasswordInput
                            label={"Passwort"}
                            description={"Das Passwort muss mindestens 8 Zeichen lang sein"}
                            placeholder={"Passwort"}
                            leftSection={<IconKey/>}
                            required
                            {...formValues.getInputProps("password")}
                        />

                        <Collapse in={formValues.values.password.length > 0} className={"stack"}>
                            <PasswordInput
                                label={"Passwort wiederholen"}
                                description={"Bitte wiederhole dein Passwort"}
                                placeholder={"Passwort wiederholen"}
                                leftSection={<IconKey/>}
                                required
                                {...formValues.getInputProps("passwordConfirm")}
                            />
                            <PasswordStrengthMeter
                                password={formValues.values.password}
                                passwordConfirm={formValues.values.passwordConfirm}
                            />
                        </Collapse>
                    </Collapse>
                </Collapse>

                <Checkbox
                    required
                    label={
                        <Text>
                            Ich habe die <Anchor
                            component={Link}
                            target={"_blank"}
                            to={"/legal/privacy-policy"}
                        >
                            Datenschutzerklärung
                        </Anchor> gelesen und akzeptiere sie.
                        </Text>
                    }
                    {...formValues.getInputProps("privacy", {type: "checkbox"})}
                />

                <Checkbox
                    required
                    label={
                        <Text>
                            Ich habe die <Anchor
                            component={Link}
                            target={"_blank"}
                            to={"/legal/terms-and-conditions"}
                        >
                            AGB der StuVe
                        </Anchor> gelesen und nehme sie zur Kenntnis
                        </Text>
                    }
                    {...formValues.getInputProps("terms", {type: "checkbox"})}
                />

                <Group>
                    <Button
                        leftSection={<IconX/>}
                        onClick={handler.close}
                        variant={"light"}
                    >
                        Abbrechen
                    </Button>

                    <Button
                        type={"submit"}
                        leftSection={<IconUserPlus/>}
                        loading={registerMutation.isPending}
                    >
                        Account anlegen
                    </Button>
                </Group>
            </form>
        </Modal>

    </>
}