import {DatePickerInput, DatePickerInputProps, DateTimePicker, DateTimePickerProps} from "@mantine/dates";
import {
    Input,
    MultiSelect,
    MultiSelectProps,
    NumberInput,
    NumberInputProps,
    Select,
    SelectProps,
    Spoiler,
    Stack,
    Text,
    Textarea,
    TextareaProps,
    TextInput,
    TextInputProps
} from "@mantine/core";
import {IconAt} from "@tabler/icons-react";
import {
    CheckboxFieldSchema,
    DateFieldSchema,
    DateRangeFieldSchema,
    EmailFieldSchema,
    FormSchemaField,
    NumberFieldSchema,
    SelectFieldSchema,
    TextFieldSchema
} from "../formBuilder/types.ts";
import {CheckboxCard, CheckboxCardProps} from "@/components/input/CheckboxCard";
import InnerHtml from "@/components/InnerHtml";
import {ReactNode} from "react";


export const FormTextField = ({field, ...props}: { field: TextFieldSchema } & TextInputProps) => {
    if (field.multiline) {
        console.error("for multiline text fields use FormTextareaField instead of FormTextField")
        return null
    }

    return <Wrapper field={field}>
        <TextInput
            required={field.required}
            placeholder={field.placeholder ?? "Text eingeben"}
            {...props}
        />
    </Wrapper>

}
export const FormTextareaField = ({field, ...props}: { field: TextFieldSchema } & TextareaProps) => {
    if (!field.multiline) {
        console.error("for single line text fields use FormTextField instead of FormTextareaField")
        return null
    }
    return <Wrapper field={field}>
        <Textarea
            resize={"vertical"}
            required={field.required}
            placeholder={field.placeholder ?? "Text eingeben"}
            {...props}
        />
    </Wrapper>
}

export const EmailField = ({field, ...props}: { field: EmailFieldSchema } & TextInputProps) => {
    return <Wrapper field={field}>
        <TextInput
            leftSection={<IconAt/>}
            required={field.required}
            placeholder={field.placeholder ?? "Email eingeben"}
            type={"email"}
            {...props}
        />
    </Wrapper>
}

export const NumberField = ({field, ...props}: { field: NumberFieldSchema } & NumberInputProps) => {

    return <Wrapper field={field}>
        <NumberInput
            required={field.required}
            placeholder={field.placeholder ?? "Zahl eingeben"}
            {...props}
        />
    </Wrapper>
}

export const DateField = ({field, ...props}: { field: DateFieldSchema } & DateTimePickerProps) => {
    return <Wrapper field={field}>
        <DateTimePicker
            clearable
            required={field.required}
            placeholder={field.placeholder ?? "Datum auswählen"}
            {...props}
        />
    </Wrapper>
}

export const DateRangeField = ({field, ...props}: { field: DateRangeFieldSchema } & DatePickerInputProps<"range">) => {
    return <Wrapper field={field}>
        {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-ignore
            <DatePickerInput
                type="range"
                clearable
                required={field.required}
                placeholder={field.placeholder ?? "Zeitraum auswählen"}
                {...props}
            />
        }
    </Wrapper>
}

export const SelectField = ({field, ...props}: {
    field: SelectFieldSchema
} & SelectProps & MultiSelectProps) => {

    const Component = field.multiple ? MultiSelect : Select

    return <>
        <Wrapper field={field}>
            <Component
                clearable
                required={field.required}
                placeholder={field.placeholder ?? "Auswählen"}
                {...props}
                data={field.options}
            />
        </Wrapper>
    </>
}

const Wrapper = ({field, children}: {
    field: FormSchemaField,
    children: ReactNode
}) => {
    return <Stack gap={5}>
        <Input.Label required>{field.label}</Input.Label>

        {field.description && <>
            <Input.Description>
                <Spoiler
                    maxHeight={40}
                    showLabel={<Text span size={"xs"}>Mehr anzeigen</Text>}
                    hideLabel={<Text span size={"xs"}>Weniger anzeigen</Text>}
                >
                    <InnerHtml html={field.description}/>
                </Spoiler>
            </Input.Description>
        </>}

        {children}
    </Stack>
}


export const CheckboxField = ({field, ...props}: { field: CheckboxFieldSchema } & CheckboxCardProps) => {
    return <CheckboxCard
        label={field.label}
        description={field.description}
        required={field.required}
        {...props}
    />
}