import {FormSchema} from "../formBuilder/types.ts";
import {Button, Group,} from "@mantine/core";
import {useForm} from "@mantine/form";

import ShowDebug from "../../ShowDebug.tsx";
import {FieldEntries} from "@/components/formUtil/FromInput/types.ts";
import {createValidationFromSchema} from "@/components/formUtil/FromInput/validation.ts";
import {
    CheckboxField,
    DateField, DateRangeField,
    EmailField,
    FormTextareaField,
    FormTextField,
    NumberField, SelectField
} from "@/components/formUtil/FromInput/formFieldComponents.tsx";

/**
 * This function creates default values based on a data type (e.g. "" for text, false for checkbox)
 *
 * If initialEntries are provided, only the missing fields will be created, this appears to be useful for updating
 * existing entries
 *
 * @param schema The schema to create the default values from
 * @param initialEntries The already existing entries
 */
const createDefaultValuesFromSchema = (schema: FormSchema, initialEntries?: FieldEntries): FieldEntries => {
    const entries: FieldEntries = initialEntries ?? {}

    schema.fields.forEach(field => {
        // if the field already exists, skip it
        if (entries[field.id]) return

        // if the field does not exist, create it
        switch (field.dataType) {
            case "text":
                entries[field.id] = {
                    value: "",
                    dataType: field.dataType,
                }
                break
            case "email":
                entries[field.id] = {
                    value: "",
                    dataType: field.dataType,
                }
                break
            case "number":
                entries[field.id] = {
                    value: null,
                    dataType: field.dataType,
                }
                break
            case "checkbox":
                entries[field.id] = {
                    value: false,
                    dataType: field.dataType,
                }
                break
            case "select":
                entries[field.id] = {
                    value: field.multiple ? [] : null,
                    dataType: field.dataType,
                }
                break
            case "date":
                entries[field.id] = {
                    value: null,
                    dataType: field.dataType,
                }
                break
            case "date-range":
                entries[field.id] = {
                    value: [null, null],
                    dataType: field.dataType,
                }
        }
    })

    return entries
}


/**
 * This renders a form based on a schema
 * @see FormBuilder
 * @param schema The schema to render
 * @param onAbort The function to call when the form is aborted
 * @param onSubmit The function to call when the form is submitted
 * @param disabled Whether the form is disabled
 * @param initialData The initial data to populate the form with
 */
export default function FormInput({schema, onAbort, onSubmit, disabled, initialData}: {
    schema: FormSchema,
    onAbort?: () => void,
    onSubmit?: (values: FieldEntries) => Promise<unknown>,
    disabled?: boolean,
    initialData?: FieldEntries
}) {
    const validationSchema = createValidationFromSchema(schema)

    const formValues = useForm({
        initialValues: createDefaultValuesFromSchema(schema, initialData),
        validate: validationSchema
    })

    return <form
        className={"stack"}
        onSubmit={formValues.onSubmit((values) => {
            if (disabled) return
            onSubmit?.(values).then(() => {
                formValues.reset()
            })
        })}
    >
        {schema.fields.map(field => {
            let Component = null

            switch (field.dataType) {
                case "text":
                    Component = field.multiline ? FormTextareaField : FormTextField
                    break

                case "email":
                    Component = EmailField
                    break

                case "number":
                    Component = NumberField
                    break

                case "date":
                    Component = DateField
                    break

                case "date-range":
                    Component = DateRangeField
                    break

                case "select":
                    Component = SelectField
                    break

                case "checkbox":
                    Component = CheckboxField
                    break
            }
            return <Component
                key={field.id}
                field={field as never}
                {...formValues.getInputProps(`${field.id}.value`, {
                    type: field.dataType === "checkbox" ? "checkbox" : undefined,
                })}
                error={formValues.errors[field.id]}
            />
        })}

        <Group>
            {
                onAbort &&
                <Button disabled={disabled} onClick={onAbort} variant={"light"}>
                    Abbrechen
                </Button>
            }

            <Button disabled={disabled} type={"submit"}>Speichern</Button>
        </Group>

        <ShowDebug>
            <pre>
                {JSON.stringify(formValues, null, 2)}
            </pre>
        </ShowDebug>
    </form>
}