import {
    DateFieldSchema,
    EmailFieldSchema,
    FormSchema,
    FormSchemaField,
    NumberFieldSchema,
    TextFieldSchema
} from "../formBuilder/types.ts"
import {pprintDateTime} from "@/lib/datetime.ts";
import {FieldEntry} from "@/components/formUtil/FromInput/types.ts";


const textValidator = (field: TextFieldSchema, value: string) => {

    // check min length
    if (field.validator.minLength && value.length < field.validator.minLength) {
        return `Muss mindestens ${field.validator.minLength} Zeichen lang sein`
    }

    // check max length
    if (field.validator.maxLength && value.length > field.validator.maxLength) {
        return `Darf nicht länger als ${field.validator.maxLength} Zeichen lang sein`
    }

    // check regex
    if (field.validator.regex && !new RegExp(field.validator.regex).test(value)) {
        return "Invalid format"
    }

    return null
}

const emailValidator = (field: EmailFieldSchema, value: string) => {

    // return if no email
    if (!value) return null

    // match the email address with the regex (with named domain group)
    const emailRegex = new RegExp(/^[a-zA-Z0-9._%+-]+@(?<domain>[a-zA-Z0-9.-]+\.[a-zA-Z]{2,})$/);
    const match = value.match(emailRegex);

    // check if the email is valid
    if (!match) {
        return "Invalide E-Mail Adresse";
    }

    // check if the domain is allowed, use domain regex group to match the domain, ignore case
    const emailDomain = match.groups!.domain.toLowerCase();
    if (field.validator?.allowedDomains?.length > 0 && !field.validator.allowedDomains.some(domain => domain.toLowerCase() === emailDomain)) {
        return "Ungültige Domain";
    }

    return null
}

const numberValidator = (field: NumberFieldSchema, value: number) => {

    // check if the value is a number
    if (isNaN(value)) {
        return "Muss eine Zahl sein";
    }

    // check min
    if (field.validator.min && value < field.validator.min) {
        return `Muss mindestens ${field.validator.min} sein`;
    }

    // check max
    if (field.validator.max && value > field.validator.max) {
        return `Darf nicht größer als ${field.validator.max} sein`;
    }

    return null;
}

const dateValidator = (field: DateFieldSchema, value: Date) => {

    // check min date
    if (field.validator.minDate && value < field.validator.minDate) {
        return `Muss nach ${pprintDateTime(field.validator.minDate)} sein`;
    }

    // check max date
    if (field.validator.maxDate && value > field.validator.maxDate) {
        return `Muss vor ${pprintDateTime(field.validator.maxDate)} sein`;
    }

    return null;
}


const createFieldValidation = (field: FormSchemaField) => (entry?: FieldEntry) => {

    // check if required field is empty
    if (field.required && (
        !entry || !entry.value && entry.value !== 0 ||
        Array.isArray(entry.value) && entry.value.filter(i => !!i).length == 0
    )) {
        return "Das Feld darf nicht leer sein"
    }

    // check if entry is undefined
    if (!entry) {
        return null
    }

    switch (field.dataType) {
        case "text":
            return textValidator(field, entry.value as string)
        case "email":
            return emailValidator(field, entry.value as string)
        case "number":
            return numberValidator(field, entry.value as number)
        case "date":
            return dateValidator(field, entry.value as Date)
        case "date-range":
        case "select":
        case "checkbox":
            return null // no additional validation needed
    }
}


export const createValidationFromSchema = (schema: FormSchema) => {
    return schema.fields.reduce((acc, field) => ({
        [field.id]: createFieldValidation(field),
        ...acc
    }), {} as Record<string, (value: FieldEntry) => string | null>)

}