import {EmailFieldSchema, FormSchema, FormSchemaField, SelectFieldSchema} from "./types.ts";
import {UseFormReturnType} from "@mantine/form";
import {
    ActionIcon,
    Alert,
    Badge,
    Button,
    Checkbox,
    Fieldset,
    Group,
    NumberInput,
    Spoiler,
    Stack,
    Text,
    TextInput,
    ThemeIcon,
    Tooltip
} from "@mantine/core";
import {
    IconDragDrop,
    IconGrid3x3,
    IconInfoCircle,
    IconPlus,
    IconSettings,
    IconSettingsOff,
    IconTrash,
    IconTrashOff,
    IconX
} from "@tabler/icons-react";
import FormTypeIcon from "../FormTypeIcon.tsx";
import {DateTimePicker} from "@mantine/dates";
import {useDisclosure} from "@mantine/hooks";

import classes from "./editField.module.css"
import {Draggable} from "@hello-pangea/dnd";
import {humanReadableField} from "../util.ts";
import TextEditor from "@/components/input/Editor";
import InnerHtml from "@/components/InnerHtml";

type EditFieldProps = {
    index: number;
    field: FormSchemaField;
    formValues: UseFormReturnType<FormSchema>;
}

/**
 * Validation settings for text fields
 * Min length, max length and regex
 */
const TextFieldSettings = ({formValues, index}: Omit<EditFieldProps, "field">) => {
    return <>
        <NumberInput
            label={"Minimale Zeichenanzahl"}
            placeholder={"unbegrenzt"}
            {...formValues.getInputProps(`fields.${index}.validator.minLength`)}
        />

        <NumberInput
            label={"Maximale Zeichenanzahl"}
            placeholder={"unbegrenzt"}
            {...formValues.getInputProps(`fields.${index}.validator.maxLength`)}
        />

        <TextInput
            classNames={{
                input: "monospace"
            }}
            label={"Regex (für Nerds)"}
            placeholder={"/.*/gm"}
            {...formValues.getInputProps(`fields.${index}.validator.regex`)}
        />
    </>
}

/**
 * Validation settings for date fields
 * Min date and max date
 */
const DateFieldSettings = ({formValues, index}: Omit<EditFieldProps, "field">) => {
    return <>
        <DateTimePicker
            label={"Minimales Datum"}
            placeholder={"unbegrenzt"}
            {...formValues.getInputProps(`fields.${index}.validator.minDate`)}
        />

        <DateTimePicker
            label={"Maximales Datum"}
            placeholder={"unbegrenzt"}
            {...formValues.getInputProps(`fields.${index}.validator.maxDate`)}
        />
    </>
}

/**
 * Validation settings for number fields
 * Min number and max number
 */
const NumberFieldSettings = ({formValues, index}: Omit<EditFieldProps, "field">) => {
    return <>
        <NumberInput
            label={"Minimale Zahl"}
            placeholder={"unbegrenzt"}
            {...formValues.getInputProps(`fields.${index}.validator.min`)}
        />

        <NumberInput
            label={"Maximale Zahl"}
            placeholder={"unbegrenzt"}
            {...formValues.getInputProps(`fields.${index}.validator.max`)}
        />
    </>
}

/**
 * Settings for select fields
 * List to add and remove options
 */
const SelectFieldSettings = ({field, formValues, index}: Omit<EditFieldProps, "field"> & {
    field: SelectFieldSchema
}) => {

    const error = formValues.errors?.[`fields.${index}.options`]

    return <>
        <Fieldset legend={"Auswahloptionen"} className={"stack"}>

            {error && <Text size={"sm"} c={"red"}>{error}</Text>}

            {field.options.map((_, optionIndex: number) => {
                return <TextInput
                    key={optionIndex}
                    w={"100%"}
                    variant="filled"
                    placeholder={"Option"}

                    rightSection={
                        <ActionIcon
                            disabled={field.meta?.required}
                            color={"red"}
                            variant={"transparent"}
                            aria-label={"delete option"}
                            onClick={() => {
                                formValues.removeListItem(`fields.${index}.options`, optionIndex)
                            }}
                        >
                            <IconX/>
                        </ActionIcon>
                    }

                    {...formValues.getInputProps(`fields.${index}.options.${optionIndex}`)}
                    error={!!error}
                />
            })}

            <Group>
                <Button
                    disabled={field.meta?.required}
                    variant={"light"} size="xs"
                    leftSection={<IconPlus/>}
                    onClick={() => {
                        formValues.insertListItem(`fields.${index}.options`, "")
                    }}
                >
                    Option hinzufügen
                </Button>
            </Group>
        </Fieldset>
    </>
}

/**
 * Settings for email fields
 * List to add and remove allowed domains
 */
const EmailFieldSettings = ({field, formValues, index}: Omit<EditFieldProps, "field"> & {
    field: EmailFieldSchema
}) => {
    return <>
        <Fieldset legend={"Erlaubte Email Domain's"} className={"stack"}>

            {
                (field.validator?.allowedDomains || []).length == 0 && (
                    <Text size={"sm"} c={"green"}>
                        Alle Email Adressen sind erlaubt
                    </Text>
                )
            }

            {(field.validator?.allowedDomains ?? []).map((_, optionIndex: number) => {
                return <TextInput
                    key={optionIndex}
                    w={"100%"}
                    variant="filled"
                    placeholder={"example.com"}

                    rightSection={
                        <ActionIcon
                            color={"red"}
                            variant={"transparent"}
                            aria-label={"delete option"}
                            onClick={() => {
                                formValues.removeListItem(`fields.${index}.validator.allowedDomains`, optionIndex)
                            }}
                        >
                            <IconX/>
                        </ActionIcon>

                    }

                    {...formValues.getInputProps(`fields.${index}.validator.allowedDomains.${optionIndex}`)}
                />
            })}

            <Group>
                <Button
                    variant={"light"} size="xs"
                    leftSection={<IconPlus/>}
                    onClick={() => {
                        formValues.insertListItem(`fields.${index}.validator.allowedDomains`, "")
                    }}
                >
                    Domain hinzufügen
                </Button>
            </Group>
        </Fieldset>
    </>
}

/**
 * This component lets the user edit a field in the form builder
 * @param field The field to edit
 * @param formValues The uncontrolled form values
 * @param index The index of the field in the form
 */
export default function EditField({field, formValues, index}: EditFieldProps) {

    // check if any errors are present for this field or its children
    const hasError = formValues.errors && Object.keys(formValues.errors).some(key => key.startsWith(`fields.${index}`))

    // toggle the expanded settings view
    const [showSettings, sowSettingsHandler] = useDisclosure(false)

    return <Draggable key={field.id} index={index} draggableId={field.id}>
        {(provided) => (
            <div ref={provided.innerRef} {...provided.draggableProps} className={classes.container}
                 data-has-error={hasError}>
                <div className={classes.header}>
                    <ThemeIcon variant={"transparent"} {...provided.dragHandleProps}>
                        <IconDragDrop/>
                    </ThemeIcon>
                    <TextInput
                        leftSection={
                            <FormTypeIcon size={16} fieldType={field.dataType}/>
                        }
                        variant="unstyled"
                        placeholder={"Name des Formularfeldes ..."}
                        required
                        disabled={field.meta?.required}
                        {...formValues.getInputProps(`fields.${index}.label`)}
                    />

                    <Badge color={"gray"} variant="light">
                        {humanReadableField(field.dataType)}
                    </Badge>

                    <ActionIcon
                        variant={"transparent"}
                        onClick={sowSettingsHandler.toggle}
                        color={hasError ? "red" : "blue"}
                    >
                        {
                            showSettings ? <IconSettingsOff/> : <IconSettings/>
                        }
                    </ActionIcon>
                </div>

                {
                    showSettings &&
                    <>
                        <Stack>

                            {field.meta?.required && (
                                <Alert color={"blue"} icon={<IconTrashOff/>}>
                                    Dieses Feld kann nicht entfernt werden, da es als benötigt festgelegt wurde.
                                </Alert>
                            )}

                            {field.meta?.description && (
                                <Alert color={"blue"} icon={<IconInfoCircle/>}>
                                    {field.meta.description}
                                </Alert>
                            )}

                            <Tooltip label={
                                "Mithilfe dieser ID kann dieses Feld referenziert werden."
                            } withArrow>
                                <Badge color={"gray"} variant="light" leftSection={<IconGrid3x3 size={12}/>}>
                                    Feld ID: {field.id}
                                </Badge>
                            </Tooltip>

                            {
                                field.meta?.required ? <>
                                    <Fieldset legend={"Beschreibung"}>
                                        <Spoiler hideLabel={"Beschreibung ausblenden"}
                                                 showLabel={"Beschreibung anzeigen"}>
                                            <InnerHtml html={field.description ?? ""}/>
                                        </Spoiler>
                                    </Fieldset>
                                </> : <TextEditor
                                    placeholder={"Beschreibung ..."}
                                    value={formValues.values.fields[index].description ?? ""}
                                    onChange={(value) => {
                                        formValues.setFieldValue(`fields.${index}.description`, value)
                                    }}
                                />
                            }

                            <Group>

                                {field.dataType !== "checkbox" && (
                                    <TextInput
                                        label={"Platzhalter"}
                                        {...formValues.getInputProps(`fields.${index}.placeholder`)}
                                    />
                                )}

                                {
                                    // validation settings for text fields
                                    (field.dataType === "text") &&
                                    <TextFieldSettings index={index} formValues={formValues}/>
                                }

                                {
                                    // validation settings for number fields
                                    (field.dataType === "number") &&
                                    <NumberFieldSettings index={index} formValues={formValues}/>
                                }

                                {
                                    // validation settings for date fields
                                    (field.dataType === "date") &&
                                    <DateFieldSettings index={index} formValues={formValues}/>
                                }
                            </Group>

                            {
                                // settings for select fields
                                (field.dataType === "select") &&
                                <SelectFieldSettings index={index} field={field} formValues={formValues}/>
                            }

                            {
                                // settings for email fields
                                (field.dataType === "email") &&
                                <EmailFieldSettings index={index} field={field} formValues={formValues}/>
                            }

                            <Group>
                                <Checkbox
                                    label={
                                        field.dataType === "checkbox" ? "Feld muss angekreuzt sein" : "Feld muss ausgefüllt werden"
                                    }
                                    disabled={field.meta?.required}
                                    {...formValues.getInputProps(`fields.${index}.required`, {type: "checkbox"})}
                                />

                                {
                                    field.dataType === "text" && <>
                                        <Checkbox
                                            label={"Mehrzeiliges Textfeld"}
                                            {...formValues.getInputProps(`fields.${index}.multiline`, {type: "checkbox"})}
                                        />
                                    </>
                                }

                                {
                                    field.dataType === "select" && <>
                                        <Checkbox
                                            label={"Mehrfachauswahl"}
                                            disabled={field.meta?.required}
                                            {...formValues.getInputProps(`fields.${index}.multiple`, {type: "checkbox"})}
                                        />
                                    </>
                                }

                                <Button
                                    ml={"auto"}
                                    variant={"light"} color={"red"} size={"xs"}
                                    leftSection={field.meta?.required ? <IconTrashOff/> : <IconTrash/>}
                                    disabled={field.meta?.required}
                                    onClick={() => {
                                        formValues.removeListItem("fields", index)
                                    }}
                                >
                                    Feld Löschen
                                </Button>
                            </Group>
                        </Stack>
                    </>
                }
            </div>
        )}
    </Draggable>
}