import {FieldDataTypes, FormSchema, FormSchemaField} from "./types.ts";
import {isNotEmpty, useForm} from "@mantine/form";
import {createNewField, humanReadableField} from "../util.ts";
import {Alert, Button, Group, Menu, Modal} from "@mantine/core";
import EditField from "./editField.tsx";
import {IconDatabaseOff, IconEye, IconSquarePlus2} from "@tabler/icons-react";
import FormTypeIcon from "../FormTypeIcon.tsx";
import {useDisclosure} from "@mantine/hooks";
import FormInput from "../FromInput";
import {DragDropContext, Droppable} from "@hello-pangea/dnd";
import {useEffect} from "react";

/**
 * FormBuilder
 * This component is used to create a form schema by adding fields to it.
 * The form schema can then be used to render a form with FormInput.
 * @see FormInput
 * @param defaultValue The default value of the form schema
 * @param withPreview Whether to show a preview of the form
 * @param onSubmit The function to call when the for schema is submitted
 * @param reservedFieldLabels A list of reserved field labels that cannot be used
 * @param additionalSchemaToPreview Additional schema to show in the preview, only used if withPreview is true
 * @constructor
 */
export default function FormBuilder({
                                        defaultValue,
                                        withPreview,
                                        onSubmit,
                                        reservedFieldLabels,
                                        additionalSchemaToPreview
                                    }: {
    defaultValue: FormSchema,
    onSubmit: (values: FormSchema) => void,
    withPreview?: boolean,
    additionalSchemaToPreview?: FormSchema,
    reservedFieldLabels?: string[]
}) {

    const [showPreview, setShowPreview] = useDisclosure(false)

    const formValues = useForm({
            initialValues: defaultValue,
            validateInputOnChange: true,
            validate: {
                fields: {
                    // check if label is unique
                    label: (value, values) => {

                        // check if label is unique
                        if (values.fields.map(({label}) => label).filter(isNotEmpty).filter((label) => label === value).length > 1) {
                            return "Der Name muss eindeutig sein"
                        }

                        // check if label is empty
                        if (value.length == 0) {
                            return "Der Name darf nicht leer sein"
                        }

                        // check if label is in reservedFieldNames
                        if (reservedFieldLabels?.includes(value)) {
                            return "Der Name ist reserviert"
                        }

                        return null
                    },

                    // check if all options are unique and not empty
                    options: value => {
                        if (value === undefined) {
                            return null
                        }

                        if (value.length === 0) {
                            return "Es muss mindestens eine Option geben"
                        }

                        if (value?.some((option) => option === "")) {
                            return "Optionen dürfen nicht leer sein"
                        }

                        const isUnique = value && (value.length === new Set(value).size)
                        return isUnique ? null : "Alle Optionen müssen eindeutig sein"
                    }
                }
            }
        }
    )

    useEffect(() => {
        formValues.setValues(defaultValue)
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [defaultValue])

    return (
        <form className={"stack"} onSubmit={formValues.onSubmit(values => onSubmit(values))}>
            <DragDropContext
                onDragEnd={({destination, source}) =>
                    destination?.index !== undefined && formValues.reorderListItem('fields', {
                        from: source.index,
                        to: destination.index
                    })
                }
            >
                <Droppable droppableId="dnd-list" direction="vertical">
                    {(provided) => (
                        <div  {...provided.droppableProps} ref={provided.innerRef}>
                            {formValues.values.fields.map((field: FormSchemaField, index: number) => {
                                return <EditField key={field.id} index={index} field={field} formValues={formValues}/>
                            })}
                            {provided.placeholder}
                        </div>
                    )}
                </Droppable>
            </DragDropContext>
            <Group>
                <Menu position="bottom-start" withArrow>
                    <Menu.Target>
                        <Button leftSection={<IconSquarePlus2 size={16}/>} variant={"light"}>
                            Formularfeld hinzufügen
                        </Button>
                    </Menu.Target>
                    <Menu.Dropdown>
                        {
                            FieldDataTypes.map((fieldType) => {
                                return <Menu.Item
                                    leftSection={<FormTypeIcon size={16} fieldType={fieldType}/>}
                                    onClick={() => formValues.insertListItem("fields", createNewField(fieldType))}
                                    key={fieldType}
                                >
                                    {humanReadableField(fieldType)}
                                </Menu.Item>
                            })
                        }
                    </Menu.Dropdown>
                </Menu>

                {
                    withPreview && <>
                        <Modal opened={showPreview} onClose={setShowPreview.close} title={"Vorschau"} size={"lg"}>
                            <div className={`stack`}>
                                <Alert color={"orange"} icon={<IconDatabaseOff/>}>
                                    Die Vorschau zeigt nur die Struktur des Formulars an.
                                    <br/>
                                    Die Eingaben werden nicht gespeichert.
                                </Alert>
                                <FormInput schema={{
                                    fields: [
                                        ...formValues.values.fields,
                                        ...(additionalSchemaToPreview?.fields ?? [])
                                    ]
                                }} disabled/>
                            </div>
                        </Modal>

                        <Button
                            onClick={setShowPreview.toggle}
                            variant={"light"}
                            leftSection={<IconEye size={16}/>}
                        >
                            Vorschau
                        </Button>

                        <Button type={"submit"}>
                            Speichern
                        </Button>
                    </>
                }
            </Group>
        </form>
    )
}