import {FieldEntry} from "@/components/formUtil/FromInput/types.ts";
import {ReactNode} from "react";
import {Anchor, Badge, Code, Group, List, Spoiler, ThemeIcon, Tooltip} from "@mantine/core";
import {
    IconCalendar,
    IconCheck,
    IconCheckbox,
    IconCircleOff,
    IconClock,
    IconEye,
    IconEyeOff,
    IconX
} from "@tabler/icons-react";
import {areDatesSame, formatDuration, pprintDateTime} from "@/lib/datetime.ts";
import dayjs from "dayjs";

const EmptyCell = () => {
    return <>
        <Tooltip label={"Nicht angeben"} withArrow>
            <Badge
                leftSection={<IconCircleOff size={10}/>}
                variant="light" color="gray"
            >
                N/A
            </Badge>
        </Tooltip>
    </>
}

const DateTimeCell = ({date}: { date: Date | string }) => {
    return <>
        {pprintDateTime(date)}
    </>
}

const DateRangeCell = ({dateRange}: { dateRange: [Date | null, Date | null] }) => {

    const [start, end] = dateRange

    if (start === null || end === null) {
        return <EmptyCell/>
    }

    const duration = formatDuration(start, end)

    // case for same date
    if (areDatesSame(start, end)) {
        return <Tooltip label={duration} withArrow>
            <Group gap={"xs"} justify={"center"}>
                <Group wrap={"nowrap"} gap={2} align={"start"}>
                    <IconClock size={16}/>
                    {dayjs(start).format("HH:mm")}
                    {"-"}
                    {dayjs(end).format("HH:mm")}
                </Group>
                <Group gap={2} wrap={"nowrap"} align={"start"}>
                    <IconCalendar size={16}/>
                    {dayjs(start).format("DD.MM.YY")}
                </Group>
            </Group>
        </Tooltip>
    }

    // case both dates start at 00:00:00
    if (start.getHours() === 0 && start.getMinutes() === 0 && start.getSeconds() === 0 &&
        end.getHours() === 0 && end.getMinutes() === 0 && end.getSeconds() === 0) {
        return <Tooltip label={duration} withArrow>
            <Group gap={'xs'} justify={"center"}>
                <Group gap={2} wrap={"nowrap"} align={"start"}>
                    <IconCalendar size={16}/>
                    {dayjs(start).format("DD.MM.YY")}
                </Group>
                {"-"}
                <Group gap={2} wrap={"nowrap"} align={"start"}>
                    <IconCalendar size={16}/>
                    {dayjs(end).format("DD.MM.YY")}
                </Group>
            </Group>
        </Tooltip>
    }

    // case different dates and times
    return <Tooltip label={duration} withArrow>
        <Group gap={'xs'} justify={"center"}>
            <Group gap={2} wrap={"nowrap"} align={"start"}>
                <IconClock size={16}/>
                {dayjs(start).format("HH:mm")}
                <IconCalendar size={16}/>
                {dayjs(start).format("DD.MM.YY")}
            </Group>
            {"bis"}
            <Group gap={2} wrap={"nowrap"} align={"start"}>
                <IconClock size={16}/>
                {dayjs(end).format("HH:mm")}
                <IconCalendar size={16}/>
                {dayjs(end).format("DD.MM.YY")}
            </Group>
        </Group>
    </Tooltip>
}

const NumberCell = ({value}: { value: number }) => {
    return <Code>{value}</Code>
}

const MailCell = ({value}: { value: string }) => {
    return <Anchor href={`mailto:${value}`}>{value}</Anchor>
}

const TextCell = ({value}: { value: string }) => {
    return <>
        <Spoiler maxHeight={120}
                 showLabel={
                     <ThemeIcon size={"sm"} variant={"transparent"} aria-label={"show more"}>
                         <IconEye/>
                     </ThemeIcon>
                 }
                 hideLabel={
                     <ThemeIcon size={"sm"} variant={"transparent"} aria-label={"show less"}>
                         <IconEyeOff/>
                     </ThemeIcon>
                 }
        >
            {value}
        </Spoiler>
    </>
}

const BooleanCell = ({value}: { value: boolean }) => {
    return (
        <Badge
            variant={"light"}
            leftSection={value ? <IconCheck size={16}/> : <IconX size={16}/>}
            color={value ? "green" : "red"}
        >
            {value ? "Ja" : "Nein"}
        </Badge>
    )
}

const SelectCell = ({value}: { value: string | string[] }) => {
    return <>
        <List size={"sm"} icon={<IconCheckbox size={10}/>}>
            {
                (Array.isArray(value) ? value : [value]).map((item, index) => (
                    <List.Item key={index}>
                        {item}
                    </List.Item>
                ))
            }
        </List>
    </>
}

export default function RenderCell({entry}: { entry?: FieldEntry }): ReactNode {

    if (!entry) {
        return <EmptyCell/>
    }

    const {value, dataType} = entry

    if (value === undefined ||
        value === null ||
        value === '') {
        return <EmptyCell/>
    }

    switch (dataType) {
        case "date":
            return <DateTimeCell date={value}/>
        case "date-range":
            return <DateRangeCell dateRange={value}/>
        case "number":
            return <NumberCell value={value as number}/>
        case "email":
            return <MailCell value={value}/>
        case "text":
            return <TextCell value={value}/>
        case "checkbox":
            return <BooleanCell value={value}/>
        case "select":
            return <SelectCell value={value}/>
    }
}
