import {BubbleMenu, Editor, useEditor} from "@tiptap/react";
import {StarterKit} from "@tiptap/starter-kit";
import {Underline} from "@tiptap/extension-underline";
import Placeholder from '@tiptap/extension-placeholder';
import {Link, RichTextEditor, RichTextEditorContent} from "@mantine/tiptap";
import classes from './index.module.css';
import {Box, Input, InputWrapperProps, Loader} from "@mantine/core";
import {useEffect} from "react";
import sanitizeHtml from 'sanitize-html';


const Bubble = ({editor}: { editor: Editor }) => (
    <BubbleMenu editor={editor}>
        <RichTextEditor.ControlsGroup>
            <RichTextEditor.Bold/>
            <RichTextEditor.Italic/>
        </RichTextEditor.ControlsGroup>
    </BubbleMenu>
)

const Toolbar = ({fullToolbar}: { fullToolbar: boolean, editor: Editor }) => (
    <RichTextEditor.Toolbar>
        {
            fullToolbar ?
                <>
                    <RichTextEditor.ControlsGroup>
                        <RichTextEditor.Bold/>
                        <RichTextEditor.Italic/>
                        <RichTextEditor.Underline/>
                        <RichTextEditor.Code/>
                        <RichTextEditor.Strikethrough/>
                    </RichTextEditor.ControlsGroup>

                    <RichTextEditor.ControlsGroup>
                        <RichTextEditor.H1/>
                        <RichTextEditor.H2/>
                        <RichTextEditor.H3/>
                        <RichTextEditor.H4/>
                    </RichTextEditor.ControlsGroup>

                    <RichTextEditor.ControlsGroup>
                        <RichTextEditor.Blockquote/>
                        <RichTextEditor.Hr/>
                        <RichTextEditor.BulletList/>
                        <RichTextEditor.OrderedList/>
                    </RichTextEditor.ControlsGroup>

                    <RichTextEditor.ControlsGroup>
                        <RichTextEditor.Link/>
                        <RichTextEditor.Unlink/>
                    </RichTextEditor.ControlsGroup>
                </>
                :
                <>
                    <RichTextEditor.ControlsGroup>
                        <RichTextEditor.Bold/>
                        <RichTextEditor.Italic/>
                        <RichTextEditor.Underline/>
                        <RichTextEditor.Code/>
                    </RichTextEditor.ControlsGroup>
                    <RichTextEditor.ControlsGroup>
                        <RichTextEditor.Link/>
                        <RichTextEditor.Unlink/>
                    </RichTextEditor.ControlsGroup>
                </>

        }
    </RichTextEditor.Toolbar>
)

/**
 * A wrapper around the Mantine Input component that provides a WYSIWYG editor.
 * @param value The value of the editor.
 * @param onChange The callback to call when the editor's value changes.
 * @param placeholder The placeholder text to show when the editor is empty.
 * @param fullToolbar Whether to show the full toolbar or not.
 * @param maxHeight The maximum height of the editor.
 * @param hideToolbar Whether to hide the toolbar or not. If hidden a bubble menu will be shown instead.
 * @param noBorder shows no border if true
 * @param props The props to pass to the Mantine Input Wrapper component.
 */
export default function TextEditor({
                                       value,
                                       onChange,
                                       placeholder,
                                       fullToolbar,
                                       maxHeight,
                                       hideToolbar,
                                       noBorder,
                                       disabled,
                                       ...props
                                   }: {
    value: string;
    onChange: (value: string) => void;
    placeholder?: string;
    fullToolbar?: boolean;
    maxHeight?: number | string;
    hideToolbar?: boolean;
    noBorder?: boolean;
    disabled?: boolean;
} & Omit<InputWrapperProps, "onChange">) {

    const editor = useEditor({
        extensions: [
            StarterKit,
            Underline,
            Link,
            Placeholder.configure({placeholder: placeholder})
        ],
        editable: !disabled,
        content: value,
        onUpdate: ({editor}) => {
            const cleanHtml = sanitizeHtml(editor.getHTML())
            const tempElement = document.createElement('div')
            tempElement.innerHTML = cleanHtml
            if (tempElement.textContent === '') {
                onChange('')
            } else {
                onChange(cleanHtml)
            }
        }
    })

    useEffect(() => {
        if (!editor) return
        if (editor.getHTML() === value) return
        const {from, to} = editor.state.selection
        editor.commands.setContent(value, false)
        editor.commands.setTextSelection({from, to})
    }, [editor, value])

    if (!editor) {
        return <Loader size={"xs"}/>
    }

    return (
        <Input.Wrapper miw={0} maw={"100%"} {...props}>
            <Box
                component={RichTextEditor}
                editor={editor}
                mod={{error: !!props.error, margin: !!props.label || !!props.description}}
                classNames={{
                    content: `${classes.content} scrollbar`,
                    root: `${classes.container} ${noBorder ? classes.noBorder : ''}`,
                    toolbar: classes.toolbar,
                }}
            >
                <RichTextEditorContent mah={maxHeight ?? "100px"}/>
                {hideToolbar ? <Bubble editor={editor}/> : <Toolbar editor={editor} fullToolbar={!!fullToolbar}/>}
            </Box>
        </Input.Wrapper>
    )
}