import {useDropzone} from "react-dropzone";
import {Box, BoxProps, Center, InputWrapperProps, Stack, Text, Title} from "@mantine/core";
import {IconPhoto, IconPhotoEdit, IconPhotoUp, IconPhotoX} from "@tabler/icons-react";

/**
 * This component allows the user to select images by dragging and dropping them into the component.
 * Images can also be selected by clicking on the component.
 *
 * This component allows the mime types 'image/jpeg', 'image/png' and 'image/gif'.
 *
 * @param error - An error message to display to the user.
 * @param onChange - A function that is called when the user selects images.
 * @param fileCount - The number of files that have been selected.
 * @param maxFileCount - The maximum number of files that can be selected.
 * @param small - A boolean that determines if the component should be displayed in a smaller size.
 * @param props - Additional props to pass to the component.
 */
export default function ImageSelect({
                                        error,
                                        onChange,
                                        fileCount,
                                        maxFileCount,
                                        small,
                                        ...props
                                    }: {
                                            small?: boolean
                                            onChange: (files: File[]) => void, fileCount: number, maxFileCount: number
                                        }
                                        & Omit<BoxProps, "children" | "style" | "className">
                                        & Pick<InputWrapperProps, "error">
) {
    const {
        getRootProps,
        getInputProps,
        isDragActive,
        isDragAccept,
        isDragReject
    } = useDropzone({
        accept: {
            'image/jpeg': [],
            'image/png': [],
            'image/gif': [],
        },
        maxSize: 5232880,
        onDropAccepted: (files) => onChange(files),
    })

    return (
        <>
            <Box
                className="container"
                style={(theme) => ({
                    height: "100%",
                    border: `1px dashed ${theme.colors.gray[5]}`,
                    borderColor: isDragAccept ? theme.colors.blue[5] : isDragReject ? theme.colors.red[5] : theme.colors.gray[5],
                    padding: theme.spacing.md,
                    borderRadius: theme.radius.md,
                    cursor: "pointer",
                    "&:hover": {
                        boxShadow: theme.shadows.sm,
                    }
                })}
                {...props}
            >
                <Center{...getRootProps({className: "dropzone"})} style={{height: "100%",}}>
                    <input {...getInputProps()} />
                    <Stack align="center">
                        {isDragAccept && <IconPhotoUp/>}
                        {isDragReject && <IconPhotoX/>}
                        {!isDragActive && (small ? <IconPhotoEdit/> : <IconPhoto/>)}

                        {!small && <>
                            < Title c={"green"}
                                    order={3}>{maxFileCount > 1 ? "Bilder" : "Bild"} auswählen</Title>
                            <Text size={"sm"}>Bilder können per Drag & Drop oder durch {
                                <Text span c={"green"}>klicken</Text>
                            } ausgewählt werden.</Text>
                            {maxFileCount > 1 &&
                                <Text size={"sm"} c={"dimmed"}>{fileCount} von {maxFileCount} Bildern</Text>
                            }
                            <Text size={"sm"} c={"dimmed"}>Maximale Dateigröße: 5MB - jpg/png/gif</Text>
                            {error && <Text size={"sm"} c={"red"}>{error}</Text>}
                        </>
                        }
                    </Stack>
                </Center>
            </Box>
        </>
    )
}