import {RecordModel} from "pocketbase";
import {UseMutationResult} from "@tanstack/react-query";
import {CheckIcon, Combobox, Group, Pill, PillsInput, PillsInputProps, Stack, Text, useCombobox} from "@mantine/core";
import {useState} from "react";

/*
    * GenericRecordInputProps is a generic type that describes the props that are common to all Record Input Wrappers
    * @param T - The type of the record that is being selected
    * @param selectedRecords - The records that are currently selected
    * @param setSelectedRecords - A function that sets the selected records
    * @param label - The label of the input field
    * @param description - The description of the input field
    * @param leftSection - The icon that is displayed on the left side of the input field
    * @param placeholder - The placeholder text of the input field
 */
export type GenericRecordSearchInputProps<T> = {
    selectedRecords: T[]
    setSelectedRecords: (records: T[]) => void
    placeholder?: string
} & Pick<PillsInputProps, "label" | "description" | "leftSection" | "required" | "error">

/**
 * RecordSearchInput is a generic component that can be used to create a searchable input field for selecting records from a database.
 * @param props - The props for the RecordSearchInput component
 * @param props.recordToString - A function that converts a record to a display name and description
 * @param props.recordSearchMutation - A mutation that fetches records from the database based on a search string. It is advised to use the useMutation hook from react-query.
 * @constructor
 */
export default function RecordSearchInput<T extends RecordModel>(props: {
    recordToString: (record: T) => { displayName: string, description?: string }
    recordSearchMutation: UseMutationResult<T[], Error, string, unknown>
} & GenericRecordSearchInputProps<T>) {

    const combobox = useCombobox({
        onDropdownClose: () => combobox.resetSelectedOption(),
        onDropdownOpen: () => combobox.updateSelectedOptionIndex('active'),
    })

    const [search, setSearch] = useState('')

    const selectedRecordIds = props.selectedRecords.map((record) => record.id)

    const handleValueSelect = (val: string) => {
        if (selectedRecordIds.includes(val)) {
            return handleValueRemove(val)
        }
        props.setSelectedRecords(
            [
                ...props.selectedRecords,
                props.recordSearchMutation.data?.find((record) => record.id === val)
            ].filter(record => record !== undefined) as T[]
        )
        setSearch('')
    }

    const handleValueRemove = (val: string) => {
        props.setSelectedRecords(props.selectedRecords.filter((record) => record.id !== val))
    }

    const searchResults = (props.recordSearchMutation.data || [])
        .map((recordView) => (
            <Combobox.Option value={recordView.id} key={recordView.id}
                             active={selectedRecordIds.includes(recordView.id)}>
                <Group gap="sm" wrap={"nowrap"}>
                    {selectedRecordIds.includes(recordView.id) ? <CheckIcon size={12}/> : null}

                    {props.recordToString(recordView).description ? <>
                        <Stack gap={0}>
                            <Text span size="sm">{props.recordToString(recordView).displayName}</Text>
                            <Text lineClamp={1} span c={"dimmed"}
                                  size="xs">{props.recordToString(recordView).description}</Text>
                        </Stack>
                    </> : <>
                        <Text span c={"dimmed"} size="sm">{props.recordToString(recordView).displayName}</Text>
                    </>
                    }
                </Group>
            </Combobox.Option>
        ))

    return (
        <Combobox store={combobox} onOptionSubmit={handleValueSelect}>
            <Combobox.DropdownTarget>
                <PillsInput
                    label={props.label}
                    description={props.description}
                    onClick={() => combobox.openDropdown()}
                    leftSection={props.leftSection}
                    required={props.required}
                    error={props.error}
                >
                    <Pill.Group>
                        {
                            props.selectedRecords.map((selectedRecord) => (
                                <Pill
                                    key={selectedRecord.id}
                                    withRemoveButton
                                    onRemove={() => handleValueRemove(selectedRecord.id)}
                                >
                                    {props.recordToString(selectedRecord).displayName}
                                </Pill>
                            ))
                        }

                        <Combobox.EventsTarget>
                            <PillsInput.Field
                                onFocus={() => combobox.openDropdown()}
                                onBlur={() => combobox.closeDropdown()}
                                value={search}
                                placeholder={props.placeholder}
                                onChange={(event) => {
                                    combobox.updateSelectedOptionIndex()
                                    setSearch(event.currentTarget.value)
                                    props.recordSearchMutation.mutate(event.currentTarget.value)
                                }}
                                onKeyDown={(event) => {
                                    if (event.key === 'Backspace' && search.length === 0) {
                                        event.preventDefault();
                                        handleValueRemove(props.selectedRecords[props.selectedRecords.length - 1].id)
                                    }
                                }}
                            />
                        </Combobox.EventsTarget>
                    </Pill.Group>
                </PillsInput>
            </Combobox.DropdownTarget>

            <Combobox.Dropdown>
                <Combobox.Options>
                    {searchResults.length > 0 ? searchResults :
                        <Combobox.Empty>
                            {props.recordSearchMutation.isPending ? 'Lade...' : 'Keine Ergebnisse ...'}
                        </Combobox.Empty>}
                </Combobox.Options>
            </Combobox.Dropdown>
        </Combobox>
    )
}