import dayjs, {Dayjs} from "dayjs";

import relativeTime from "dayjs/plugin/relativeTime";
import duration from "dayjs/plugin/duration";

dayjs.extend(relativeTime);
dayjs.extend(duration);

/**
 * Check if two dates are the same. This function only checks the date, not the time.
 * @param d1 date string
 * @param d2 date string
 * @return {boolean} True if the dates are the same, false otherwise.
 */
export const areDatesSame = (d1: string | Date | Dayjs, d2: string | Date | Dayjs) => {
    const date1 = dayjs(d1)
    const date2 = dayjs(d2)
    return (
        date1.year() === date2.year() &&
        date1.month() === date2.month() &&
        date1.day() === date2.day()
    )
}
/**
 * Pretty print a date. The date is formatted as "DAY DD.MM.YY".
 * @param date - The date string to pretty print.
 * @return {string} The pretty printed date.
 */
export const pprintDate = (date: string | Date | Dayjs): string => {
    const d = dayjs(date)
    return `${d.format('dd')} ${d.format('DD.MM.YY')}`
}

/**
 * Pretty print a date and time. The date is formatted as "HH:MM DAY DD.MM.YYYY".
 * Uses Dayjs
 * @param date
 */
export const pprintDateTime = (date: string | Date | Dayjs): string => {
    const d = dayjs(date)
    return `${d.format('HH:mm')} ${d.format('dd')} ${d.format('DD.MM.YY')}`
}


function getLocalizedNowWord(locale: string) {
    const translations = {
        'en-US': 'now',
        'de-DE': 'jetzt',
        'es-ES': 'ahora',
        'fr-FR': 'maintenant',
        'it-IT': 'ora',
        'ja-JP': '今',
        'zh-CN': '现在',
        'ru-RU': 'сейчас',
        'ar-SA': 'الآن'
    }
    return translations[locale as keyof typeof translations] || 'now';
}

export const humanDeltaFromNow = (start: string | Date | Dayjs, end: string | Date | Dayjs): {
    message: string,
    delta: "PAST" | "FUTURE" | "NOW"
} => {

    // check if end is in the past
    if (dayjs(end).isBefore(dayjs())) {
        return {
            message: dayjs(end).fromNow(),
            delta: "PAST"
        }
    }

    // check if start is in the future
    if (dayjs(start).isAfter(dayjs())) {
        return {
            message: dayjs(start).fromNow(),
            delta: "FUTURE"
        }
    }

    // else the event is happening now, return localized "now"
    return {
        message: getLocalizedNowWord(navigator.language),
        delta: "NOW"
    }
}

export const pprintDateRange = (d1: string | Date | Dayjs, d2: string | Date | Dayjs): string => {
    const date1 = dayjs(d1)
    const date2 = dayjs(d2)

    if (areDatesSame(date1, date2)) {
        return `${date1.format('HH:mm')} - ${date2.format('HH:mm')} ${dayjs(date2).format('DD.MM.YY')}`
    } else {
        return `${
            date1.hour() == 0 && date1.minute() == 0 && date1.second() == 0 && date1.millisecond() == 0 ?
                date1.format("dd DD.MM.YY") :
                date1.format('HH:mm DD.MM')
        } - ${
            date2.hour() == 0 && date2.minute() == 0 && date2.second() == 0 && date2.millisecond() == 0 ?
                date2.format("dd DD.MM.YY") :
                date2.format("HH:mm DD.MM")
        }`
    }
}

/**
 * Format the duration between two dates.
 * The Duration is precise to the minute.
 * Example: "3 Wo, 1 Tag, 2 Std, 53 Min"
 * @param date1
 * @param date2
 */
export function formatDuration(date1: string | Date | Dayjs, date2: string | Date | Dayjs) {
    const start = dayjs(date1)
    const end = dayjs(date2)
    const diff = end.diff(start)
    const duration = dayjs.duration(diff)
    const weeks = Math.floor(duration.asDays() / 7);
    const days = duration.days() % 7;
    const hours = duration.hours();
    const minutes = duration.minutes();

    const parts = [];

    if (weeks > 0) {
        parts.push(`${weeks} Wo`);
    }
    if (days > 0 || weeks > 0) {
        parts.push(`${days} Tag${days > 1 ? 'e' : ''}`);
    }
    if (hours > 0) {
        parts.push(`${hours} Std`);
    }
    if (minutes > 0) {
        parts.push(`${minutes} Min`);
    }

    return parts.join(', ');
}