import {createContext, DependencyList, ReactNode, useCallback, useContext, useEffect, useMemo, useState} from "react"
import PocketBase, {ClientResponseError, LocalAuthStore, RecordAuthResponse, RecordSubscription} from 'pocketbase'
import ms from "ms";
import {useInterval} from "@mantine/hooks";
import {useQuery} from "@tanstack/react-query";
import {TypedPocketBase} from "@/models";
import {PB_BASE_URL, PB_STORAGE_KEY, PB_USER_COLLECTION} from "../../config.ts";
import {UserModal} from "@/models/AuthTypes.ts";
import {Alert, List} from "@mantine/core";
import {IconAlertTriangle} from "@tabler/icons-react";
import {showSuccessNotification} from "@/components/util.tsx";


/**
 * This component displays an alert for a PocketBase error.
 * If the error is a ClientResponseError, it will display the error messages.
 * If the error is any other error, it will display the error message.
 * If there is no error, it will return null (no alert).
 * @param error The error to display
 */
export const PocketBaseErrorAlert = ({error}: { error?: Error | null }) => {

    if (error == null) {
        return null
    }

    if (error instanceof ClientResponseError) {
        return <Alert
            icon={<IconAlertTriangle/>} title={`Fehler - ${error.status}`}
            color="red" variant="light">

            {error.message}

            {error.response.data && (
                <List>
                    {Object.keys(error.response.data).map((key, index) => (
                        <List.Item key={index}>
                            {key}: {error.response.data[key].message}
                        </List.Item>
                    ))}
                </List>
            )}
        </Alert>
    }

    return <Alert
        icon={<IconAlertTriangle/>} title="Fehler"
        color="red" variant="light">
        Fehler beim Laden der Daten: {" "}
        {error.message}
    </Alert>

}

const oneMinuteInMs = ms("1 minute");

const PocketContext = createContext({})

const PocketData = () => {
    const pb = useMemo(() => {
        return new PocketBase(
            PB_BASE_URL,
            new LocalAuthStore(PB_STORAGE_KEY)
        ) as TypedPocketBase
    }, [])

    const apiIsHealthyQuery = useQuery({
        queryKey: ["apiHealthCheck"],
        queryFn: async () => {
            const res = await pb.health.check()
            return res.code === 200
        },
        refetchInterval: oneMinuteInMs
    })

    const [user, setUser] = useState(pb.authStore.model)

    pb.authStore.onChange((_, userRecord) => {
        setUser(userRecord)
    })

    const refreshUser = useCallback(async () => {
        await pb.collection(PB_USER_COLLECTION).authRefresh({
            expand: "memberOf"
        }).catch(() => {
            pb.authStore.clear()
        })
    }, [pb])

    const ldapLogin = useCallback(async (usernameOrCN: string, password: string) => {
        await pb.send<RecordAuthResponse>("/api/ldap/login", {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify({
                username: usernameOrCN,
                password: password
            })
        }).then(res => {
            pb.authStore.clear()
            pb.authStore.save(res.token, res.record)
        })
    }, [pb])

    const guestLogin = useCallback(async (usernameOrEmail: string, password: string) => {
        await pb.collection("users").authWithPassword(usernameOrEmail, password).then(res => {
            pb.authStore.clear()
            pb.authStore.save(res.token, res.record)
            console.log(res.record)
        })
    }, [pb])

    const logout = useCallback(async () => {
        pb.authStore.clear()
        showSuccessNotification("Logout erfolgreich")
    }, [pb.authStore])

    const useSubscription = <T, >({idOrName, topic = "*", callback}: {
        idOrName: string,
        topic?: string,
        callback: (data: RecordSubscription<T>) => void
    }, deps?: DependencyList | undefined) => useEffect(() => {
        pb.collection(idOrName).subscribe<T>(topic, callback)
        return () => {
            pb.collection(idOrName).unsubscribe(topic)
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, deps ? deps : []);

    useInterval(refreshUser, oneMinuteInMs)

    return {
        ldapLogin,
        guestLogin,
        logout,
        user: user as UserModal | null,
        pb,
        refreshUser,
        useSubscription,
        apiIsHealthy: apiIsHealthyQuery.data ?? false
    }
}

export const PocketBaseProvider = ({children}: {
    children: ReactNode,
    userCollection?: string
}) => {
    const data = PocketData()
    return (
        <PocketContext.Provider value={data}>
            {children}
        </PocketContext.Provider>
    )
}

// eslint-disable-next-line react-refresh/only-export-components
export const usePB = () => {
    const pb = useContext(PocketContext)
    if (!pb) {
        throw new Error("usePB must be used inside PocketBaseProvider")
    }
    return pb as ReturnType<typeof PocketData>
}