import {PB_BASE_URL} from "../../config.ts";

/**
 * This function creates a query string from an object.
 * If a value is undefined or an empty string, it will not be included in the query string.
 * @param {Object} params - The object to create the query string from.
 * @return {string} The query string.
 */
export const createQueryParams = (params: { [key: string]: string | number | boolean | undefined }): string =>
    Object.keys(params)
        .filter(key => params[key] !== undefined && params[key] !== "")
        .map(key => `${encodeURIComponent(key)}=${encodeURIComponent(params[key] as string | number | boolean)}`)
        .join('&')

/**
 * The level of error correction to use for the QR code.
 */
export type ErrorCorrectionLevel = "L" | "M" | "Q" | "H"


/**
 * Create a URL for a QR code.
 * @param options
 * @param options.data - The data to encode in the QR code.
 * @param options.ecc - The level of error correction to use for the QR code.
 * @param options.scale - The scale of the QR code.
 * @param options.border - The border of the QR code.
 * @param options.color - The color of the QR code.
 * @param options.colorBackground - The background color of the QR code.
 * @return {string} The URL for the QR code.
 */
export const createQRCodeUrl = (options: {
    data: string,
    ecc: ErrorCorrectionLevel,
    scale: number,
    border: number,
    color: string,
    colorBackground: string
}): string => {
    return `${PB_BASE_URL}/api/qr/v1?${createQueryParams(options)}`
}

export const flattenReducer = <T>(acc: T[], val: T | T[]): T[] => {
    if (Array.isArray(val)) {
        return [...acc, ...val]
    }
    return [...acc, val]
}

export const objectMap = <T extends { [key in T1]: T2 }, T1 extends string | number | symbol, T2, R>(
    obj: T,
    fn: (k: T1, v: T[T1], i: number) => R
) =>
    Object.fromEntries(
        Object.entries(obj).map(
            ([k, v], i) => [k, fn((k as T1), (v as T[T1]), i)]
        )
    )

/**
 * This functions filters out all duplicate values from an array.
 * @example [1, 2, 3, 1, 2, 4].filter(onlyUnique) // [1, 2, 3, 4]
 */
export function onlyUnique<T>(value: T, index: number, array: T[]) {
    return array.indexOf(value) === index;
}