import {Navigate, useParams} from "react-router-dom";
import {usePB} from "@/lib/pocketbase.tsx";
import {useQuery} from "@tanstack/react-query";
import {LoadingOverlay} from "@mantine/core";
import NotFound from "@/pages/not-found/index.page.tsx";
import {useEventRights} from "@/pages/events/util.ts";

/**
 * This Router loads the event and checks if the user is an admin of the event.
 * If the user is an admin, it redirects to the event admin panel, otherwise to the shared event view.
 * This is the main entry point for the event page. This way we can ensure that the user is always redirected to the correct page.
 * @constructor
 */
export default function EventNavigate() {

    const {pb} = usePB()
    const {eventId} = useParams() as { eventId: string }

    const eventQuery = useQuery({
        queryKey: ["event", eventId],
        queryFn: async () => (await pb.collection("events").getOne(eventId, {
            expand: "eventAdmins, eventListAdmins"
        }))
    })

    const {canEditEventList, canEditEvent} = useEventRights(eventQuery.data)

    if (eventQuery.isLoading) {
        return <LoadingOverlay/>
    }

    if (eventQuery.isError || !eventQuery.data) {
        return <NotFound/>
    }

    if (canEditEvent) {
        return <Navigate to={`/events/e/${eventId}`} replace/>
    }

    if (canEditEventList) {
        return <Navigate to={`/events/e/${eventId}/lists`} replace/>
    }

    return <Navigate to={`/events/s/${eventId}`} replace/>
}