import {Alert, Button, Checkbox, Divider, Grid, TextInput, Title} from "@mantine/core";
import {hasLength, useForm} from "@mantine/form";
import {DateTimePicker} from "@mantine/dates";
import {IconCheck, IconInfoCircle, IconX} from "@tabler/icons-react";
import {useMutation} from "@tanstack/react-query";
import dayjs from "dayjs";
import {UserModal} from "@/models/AuthTypes.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import UserInput from "@/components/auth/UserInput.tsx";
import {EventModel} from "@/models/EventTypes.ts";
import ShowHelp from "@/components/ShowHelp.tsx";
import {useSettings} from "@/lib/settings.ts";
import {FormSchema} from "@/components/formUtil/formBuilder/types.ts";

/**
 * This component allows the user to create a new event.
 * It only asks for the most basic information (name, start/end date, admins, isStuVeEvent) and creates the event.
 * If the event is a StuVe event, it will use the StuVe event questions as default question schema.
 * @param onSuccess - Callback that is called when the event was successfully created. The created event is passed as argument.
 * @param onAbort - Callback that is called when the user aborts the creation of the event.
 */
export default function CreateEvent({onSuccess, onAbort}: {
    onSuccess: (event: EventModel) => void,
    onAbort?: () => void
}) {
    const {user, pb} = usePB()
    const settings = useSettings()

    const stuveQuestions = JSON.parse(settings?.stuveEventQuestions?.value ?? `{ "fields":[] }`) as FormSchema

    const formValues = useForm({
        initialValues: {
            name: "",
            startDate: null,
            endDate: null,
            eventAdmins: [user] as UserModal[],
            isStuveEvent: true,
        },
        validate: {
            name: hasLength({min: 4, max: 50}, 'Der Name muss zwischen 4 und 50 Zeichen lang sein.'),
            startDate: (value) => dayjs(value).isAfter(dayjs(), "day") ? null : "Das Startdatum muss in der Zukunft liegen.",
            endDate: (value, values) => dayjs(value).isAfter(dayjs(values.startDate), "day") ? null : "Das Enddatum muss nach dem Startdatum liegen.",
            eventAdmins: (value) => value.length > 0 ? null : "Es muss mindestens ein Admin ausgewählt werden.",
        }
    })

    const createEventMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("events").create({
                ...formValues.values,
                eventAdmins: formValues.values.eventAdmins.map((member) => member.id),
                defaultEntryQuestionSchema: formValues.values.isStuveEvent ? {
                    fields: stuveQuestions.fields.map((field) => ({
                        ...field, meta: {
                            ...field.meta,
                            required: true
                        }
                    }))
                } : {fields: []}
            })
        },
        onSuccess: (data) => onSuccess(data)
    })

    return <>
        <form className={"stack"} onSubmit={formValues.onSubmit(() => createEventMutation.mutate())}>

            <Title order={2} c={"blue"}>Neues Event erstellen</Title>

            <ShowHelp>
                Hier kannst du ein neues Event erstellen. Fülle dazu die Felder aus und klicke auf "Speichern".
                <br/>
                Alle weiteren Einstellungen (z.B. die Beschreibung, Listen, ...) kannst du nach dem Erstellen des Events
                vornehmen.
                <br/>
                Du kannst auch alle Werte die du hier angibst später noch bearbeiten.
            </ShowHelp>

            <Grid>
                <Grid.Col span={{base: 12, sm: 6}}>

                    <TextInput
                        label={"Name"}
                        description={"Der Name des Events"}
                        placeholder={"Name des Events"}
                        required
                        {...formValues.getInputProps("name")}
                    />
                </Grid.Col>

                <Grid.Col span={{base: 12, sm: 6}}>
                    <UserInput
                        required
                        label={"Event-Admins"}
                        description={"Die Event-Admins können das Event bearbeiten."}
                        error={formValues.errors.eventAdmins}
                        selectedRecords={formValues.values.eventAdmins}
                        setSelectedRecords={(records) => formValues.setFieldValue("eventAdmins", records)}
                    />
                </Grid.Col>

                <Grid.Col span={{base: 12, sm: 6}}>
                    <DateTimePicker
                        label={"Startdatum"}
                        description={"Das Startdatum und Startzeit des Events"}
                        required
                        {...formValues.getInputProps("startDate")}
                    />
                </Grid.Col>
                <Grid.Col span={{base: 12, sm: 6}}>

                    <DateTimePicker
                        label={"Enddatum"}
                        description={"Das Enddatum und Endzeit des Events"}
                        required
                        {...formValues.getInputProps("endDate")}
                    />
                </Grid.Col>
            </Grid>

            <Alert variant="light" color="blue" title="StuVe Events" icon={<IconInfoCircle/>}>
                StuVe Events (z.B. Uni-Party, Fachschafts-Event, ...) werden zusammen mit der
                Studierenden Exekutive verwaltet und es gelten die AGB der Studierendenvertretung (StuVe).
                <br/>
                Externe Events (z.B. private Feiern, ...), haben eigene AGB
                und die StuVe ist nicht der Veranstalter.

                <Divider mt={"sm"} mb={"sm"}/>

                <Checkbox
                    label={"StuVe Event"}
                    {...formValues.getInputProps("isStuveEvent", {type: "checkbox"})}
                />
            </Alert>

            <PocketBaseErrorAlert error={createEventMutation.error}/>

            <div className={"group"}>
                {onAbort && (
                    <Button
                        onClick={onAbort}
                        color={"orange"}
                        leftSection={<IconX/>}
                    >
                        Abbrechen
                    </Button>
                )}

                <Button
                    color={"green"}
                    leftSection={<IconCheck/>}
                    type={"submit"}
                    loading={createEventMutation.isPending}
                >
                    Speichern
                </Button>
            </div>
        </form>
    </>
}