import {usePB} from "@/lib/pocketbase.tsx";
import {useNavigate} from "react-router-dom";
import {useState} from "react";
import dayjs from "dayjs";
import {useQuery} from "@tanstack/react-query";
import {LoadingOverlay} from "@mantine/core";
import {Calendar, dayjsLocalizer} from "react-big-calendar";

const localizer = dayjsLocalizer(dayjs)

/**
 * This component displays a calendar with events. It uses the useQuery hook to fetch events from the database.
 * The events are displayed in a react-big-calender component. The user can click on an event to navigate to the event's detail page.
 * @constructor
 */
export const EventCalendar = () => {

    const {pb} = usePB()

    const navigate = useNavigate()

    const [selectedMonth, setSelectedMonth] = useState({
        start: dayjs().startOf("month"),
        end: dayjs().endOf("month")
    })

    const eventQuery = useQuery({
        queryKey: ["events", `${selectedMonth.start} - ${selectedMonth.end}`],
        queryFn: async () => {

            const startOfMonth = selectedMonth.start.toISOString()
            const endOfMonth = selectedMonth.end.toISOString()

            const data = await pb.collection("events").getFullList({
                filter:
                    `( ` +
                    `(startDate >= "${startOfMonth}" && startDate <= "${endOfMonth}")` +
                    ` || ` +
                    `(endDate >= "${startOfMonth}" && endDate <= "${endOfMonth}")` +
                    ` ) ` +
                    ` && ` +
                    `hideFromPublic = false`,
            })

            return data.map(event => ({
                title: event.name,
                start: new Date(event.startDate),
                end: new Date(event.endDate),
                allDay: false,
                id: event.id
            }))
        }
    })

    return <>
        <div style={{position: "relative"}}>
            <LoadingOverlay visible={eventQuery.isFetching}/>

            <Calendar
                popup={true}

                onNavigate={(date) => setSelectedMonth({
                    start: dayjs(date).startOf("month"),
                    end: dayjs(date).endOf("month")
                })}

                onSelectEvent={(event) => {
                    navigate(`/events/${event.id}`)
                }}

                localizer={localizer}
                events={eventQuery.data || []}

                startAccessor="start"
                endAccessor="end"

                style={{height: 500}}
            />
        </div>
    </>
}