import {EventModel} from "@/models/EventTypes.ts";
import {useDebouncedValue, useDisclosure, useToggle} from "@mantine/hooks";
import classes from "./index.module.css";
import PBAvatar from "@/components/PBAvatar.tsx";
import {Link} from "react-router-dom";
import {areDatesSame, humanDeltaFromNow, pprintDate} from "@/lib/datetime.ts";
import {
    ActionIcon,
    Box,
    Center,
    Collapse,
    Group,
    LoadingOverlay,
    Pagination,
    rem,
    Switch,
    TextInput,
    ThemeIcon,
    Tooltip
} from "@mantine/core";
import {
    IconArchive,
    IconChevronUp,
    IconHistory,
    IconHistoryOff,
    IconInfoCircle,
    IconList,
    IconSearch,
    IconSortAscending,
    IconSortDescending,
    IconUserStar
} from "@tabler/icons-react";
import InnerHtml from "@/components/InnerHtml";
import {usePB} from "@/lib/pocketbase.tsx";
import {useState} from "react";
import {useQuery} from "@tanstack/react-query";
import dayjs from "dayjs";
import {useEventRights} from "@/pages/events/util.ts";

/**
 * This component displays a single event in a row. It displays the event's name, date, location and a collapsable description.
 * @param event
 */
const EventRow = ({event}: { event: EventModel }) => {

    const {canEditEventList, canEditEvent} = useEventRights(event)

    const [opened, handlers] = useDisclosure(false)

    const delta = humanDeltaFromNow(event.startDate, event.endDate)

    return <>
        <div className={classes.eventRow}>
            <div className={classes.row}>
                <div>
                    <PBAvatar model={event} name={event.name} img={event.img}/>
                </div>

                <div className={classes.eventCell}>
                    <Link to={`/events/${event.id}`} className={classes.eventLink}>
                        {event.name}
                    </Link>
                </div>

                <div className={classes.eventCell}>
                    {
                        areDatesSame(event.startDate, event.endDate) ?
                            pprintDate(event.startDate) : `${pprintDate(event.startDate)} - ${pprintDate(event.endDate)}`
                    }
                </div>

                <div className={classes.eventCell} data-delta={delta.delta}>
                    {delta.message}
                </div>

                <div className={classes.eventCell}>
                    {event.location}
                </div>

                <div className={classes.eventCell}>
                    {event.isStuveEvent ? "StuVe" : "Extern"}
                </div>

                <div>
                    {
                        event.eventAdmins.length === 0 ? (
                            <Tooltip label={"Dieses Event ist archiviert"} position={"left"} color={"orange"} withArrow>
                                <ThemeIcon
                                    color={"orange"}
                                    variant={"transparent"}
                                    aria-label={"event is archived"}
                                    size={"xs"}
                                    mr={"sm"}
                                >
                                    <IconArchive/>
                                </ThemeIcon>
                            </Tooltip>
                        ) : (
                            canEditEvent ? (
                                <Tooltip label={"Du bist Event Admin"} position={"left"} color={"green"} withArrow>
                                    <ThemeIcon
                                        color={"green"}
                                        variant={"transparent"}
                                        aria-label={"you are event admin"}
                                        size={"xs"}
                                        mr={"sm"}
                                    >
                                        <IconUserStar/>
                                    </ThemeIcon>
                                </Tooltip>
                            ) : canEditEventList ?
                                <Tooltip label={"Du bist Listen Admin"} position={"left"} color={"green"} withArrow>
                                    <ThemeIcon
                                        color={"green"}
                                        variant={"transparent"}
                                        aria-label={"you are list admin"}
                                        size={"xs"}
                                        mr={"sm"}
                                    >
                                        <IconList/>
                                    </ThemeIcon>
                                </Tooltip> : (
                                    <ThemeIcon
                                        variant={"transparent"}
                                        aria-label={"spacer"}
                                        size={"xs"}
                                        mr={"sm"}
                                    />
                                )
                        )
                    }

                    <Tooltip label={"Infos"} position={"left"} withArrow>
                        <ActionIcon
                            variant={"transparent"}
                            aria-label={"show info"}
                            onClick={handlers.toggle}
                            disabled={!event.description}
                        >
                            {
                                opened ? <IconChevronUp/> : <IconInfoCircle/>
                            }
                        </ActionIcon>
                    </Tooltip>
                </div>
            </div>

            <Collapse in={opened}>
                <InnerHtml className={classes.descriptionContainer} html={event.description ?? ""}/>
            </Collapse>
        </div>
    </>
}
/**
 * This component displays a list of events. It uses the useQuery hook to fetch events from the database.
 * The events are displayed in a Table component. The user can search for events and toggle the display of past events.
 * @constructor
 */
export const EventList = () => {

    const {pb} = usePB()

    const [search, setSearch] = useState("")
    const [debouncedSearch] = useDebouncedValue(search, 200);

    const [showPast, setShowPast] = useState(false)

    const [sort, toggleSort] = useToggle(['startDate', '-startDate']);

    const [activePage, setPage] = useState(1);

    const eventsQuery = useQuery({
        queryKey: ["events", `search=${debouncedSearch}`, `showPast=${showPast}`, `page=${activePage}`, `sort=${sort}`],
        queryFn: async () => {

            const filter: string[] = []

            if (search) {
                filter.push(`name ~ "${debouncedSearch}"`)
            }

            if (!showPast && !debouncedSearch) {
                filter.push(`endDate >= "${dayjs().startOf("day").toISOString()}"`)
            }

            return await pb.collection("events").getList(activePage, 10, {
                    sort: sort,
                    filter: [`hideFromPublic = false`, ...filter].join(" && ")
                }
            )
        }
    })

    return <>
        <h2>
            {
                search ? `Suchergebnisse für '${search}' (${eventsQuery.data?.totalItems ?? 0})` :
                    showPast ? `Alle Events (${eventsQuery.data?.totalItems ?? 0})` : `Events (${eventsQuery.data?.totalItems ?? 0})`
            }
        </h2>

        <Group align={"center"} mb={"sm"}>
            <TextInput
                style={{flex: 1}}
                placeholder={"Nach Events suchen"}
                aria-label="Search events"
                leftSection={<IconSearch/>}
                rightSection={
                    eventsQuery.isLoading &&
                    <Switch
                        size={"xs"}
                        mr={"md"}
                        checked={showPast}
                        onChange={(event) => setShowPast(event.currentTarget.checked)}
                        offLabel={<IconHistoryOff style={{width: rem(10), height: rem(10)}}/>}
                        onLabel={<IconHistory style={{width: rem(10), height: rem(10)}}/>}
                    />
                }
                value={search}
                onChange={(event) => setSearch(event.currentTarget.value)}
            />


            <Tooltip label={"Nach Startdatum sortieren"}>
                <ActionIcon
                    variant={"transparent"}
                    onClick={() => toggleSort()}
                    aria-label={"sort events by date"}
                    size={"sm"}
                >
                    {
                        sort === "-startDate" ? <IconSortAscending/> : <IconSortDescending/>
                    }
                </ActionIcon>
            </Tooltip>

            <Tooltip label={"Vergangene Events Anzeigen"}>
                <ActionIcon
                    variant={"transparent"}
                    onClick={() => setShowPast(!showPast)}
                    aria-label={"show past events"}
                    size={"sm"}
                >
                    {
                        showPast ? <IconHistory/> : <IconHistoryOff/>
                    }
                </ActionIcon>
            </Tooltip>
        </Group>


        <Box
            style={{position: "relative"}}
            mb={"sm"}
        >
            <LoadingOverlay visible={eventsQuery.isFetching}/>


            <div className={classes.eventsList}>
                {eventsQuery.data?.items.map(event => {
                    return <EventRow event={event} key={event.id}/>
                })}
            </div>
        </Box>

        <Center>
            <Pagination value={activePage} onChange={setPage} total={eventsQuery.data?.totalPages ?? 10}/>
        </Center>
    </>
}