import {Link, Navigate, NavLink, Outlet, Route, Routes, useMatch, useParams} from "react-router-dom";
import {usePB} from "@/lib/pocketbase.tsx";
import {useQuery} from "@tanstack/react-query";
import {
    ActionIcon,
    ActionIconProps,
    Alert,
    Anchor,
    Breadcrumbs,
    createPolymorphicComponent,
    Grid,
    Group,
    LoadingOverlay,
    ThemeIcon,
    Title,
    Tooltip
} from "@mantine/core";
import PBAvatar from "@/components/PBAvatar.tsx";

import classes from "./EditEventRouter.module.css";
import {
    IconArchive,
    IconExternalLink,
    IconInfoCircle,
    IconList,
    IconPencil,
    IconQrcode,
    IconSectionSign,
    IconSettings
} from "@tabler/icons-react";
import NotFound from "@/pages/not-found/index.page.tsx";
import EventDescription from "./EventComponents/EventDescription.tsx";
import EventAGB from "./EventComponents/EventAGB.tsx";
import EventData from "./EventComponents/EventData.tsx";
import EventSettingsRouter, {
    EventSettingsMenu
} from "@/pages/events/e/:eventId/EventComponents/EventSettings/EventSettingsRouter.tsx";
import EventLinks from "./EventComponents/EventLinks.tsx";
import EventListsRouter, {EventListsMenu} from "./EventLists/EventListsRouter.tsx";
import {useEventRights} from "@/pages/events/util.ts";
import EventFavourites from "./EventComponents/EventFavourites.tsx";
import {forwardRef} from "react";
import {APP_URL} from "../../../../../config.ts";


type NavIconProps = {
    isActive?: boolean
    label: string
} & ActionIconProps

const NavIcon = createPolymorphicComponent<typeof ActionIcon, NavIconProps>(
    forwardRef<HTMLButtonElement, NavIconProps>((
        props,
        ref
    ) => {
        const {isActive, ...rest} = props;
        return (
            <Tooltip label={props.label} color={isActive ? 'green' : 'blue'}>
                <ActionIcon
                    ref={ref}
                    radius="xl"
                    color={isActive ? 'green' : 'blue'}
                    variant={isActive ? 'outline' : 'subtle'}
                    size="lg"
                    aria-label={props.label}
                    {...rest}
                >
                    {props.children}
                </ActionIcon>
            </Tooltip>
        )
    })
)

/**
 * Home Layout for an Event Admin Panel
 * @constructor
 */
export default function EditEventRouter() {

    const {pb} = usePB()
    const {eventId} = useParams() as { eventId: string }
    const settingsPathMath = useMatch(`/events/e/${eventId}/settings/*`);
    const listsPathMath = useMatch(`/events/e/${eventId}/lists/*`);

    const eventQuery = useQuery({
        queryKey: ["event", eventId],
        queryFn: async () => (await pb.collection("events").getOne(eventId, {
            expand: "eventAdmins, eventListAdmins"
        }))
    })

    const {canEditEventList, canEditEvent} = useEventRights(eventQuery.data)

    if (eventQuery.isLoading) {
        return <LoadingOverlay/>
    }

    if (eventQuery.isError || !eventQuery.data) {
        return <NotFound/>
    }

    if (!(canEditEvent || canEditEventList)) {
        return <Navigate to={`/events/s/${eventId}`} replace/>
    }

    const event = eventQuery.data

    const readOnlyEvent = event.eventAdmins.length == 0

    return <>
        <div className={"section-transparent"}>
            <Breadcrumbs>{[
                {title: "Home", to: "/"},
                {title: "Events", to: "/events"},
                {title: event.name, to: `/events/${event.id}`},
            ].map(({title, to}) => (
                <Anchor component={Link} to={to} key={title}>
                    {title}
                </Anchor>
            ))}</Breadcrumbs>
        </div>

        {
            readOnlyEvent && <Alert
                variant="light"
                color="orange"
                title="Archives Event"
                icon={<IconArchive/>}
                className={"section-transparent"}
                p={"sm"}
            >
                Dieses Event ist archiviert und wird nicht mehr verwaltet.
            </Alert>
        }

        <div className={"section"}>
            <PBAvatar className={"section-icon"} model={event} name={event.name}
                      img={event.img}/>
            <Title order={1} c={"green"} style={{overflowWrap: "break-word"}}>
                <ThemeIcon me={"sm"} variant="light" radius={"xl"} size="xl" color="green">
                    <IconPencil/>
                </ThemeIcon>
                {event.name}
            </Title>
        </div>

        <Grid className={classes.grid} gutter={"sm"}>
            <Grid.Col span={{base: 12, sm: 3}}>
                <div className={"stack"}>
                    <EventData event={event}/>
                    <EventLinks event={event}/>
                    <EventFavourites event={event}/>
                </div>
            </Grid.Col>

            <Grid.Col span={{base: 12, sm: 9}} className={"stack"}>
                <Group gap={"sm"} align={"center"}>
                    <NavLink to={`/events/e/${eventId}/description`} replace className={classes.navLink}>
                        {({isActive}) => (
                            <NavIcon
                                isActive={isActive}
                                component={"span"}
                                label={"Event Beschreibung"}
                            >
                                <IconInfoCircle/>
                            </NavIcon>
                        )}
                    </NavLink>

                    {canEditEvent && (
                        <EventSettingsMenu event={event} target={<NavIcon
                            isActive={!!settingsPathMath}
                            label={"Event Einstellungen"}
                        >
                            <IconSettings/>
                        </NavIcon>}/>
                    )}

                    <EventListsMenu event={event} target={<NavIcon
                        isActive={!!listsPathMath}
                        label={"Event Listen"}
                    >
                        <IconList/>
                    </NavIcon>}/>

                    <NavLink to={`/events/e/${eventId}/terms-and-conditions`} replace className={classes.navLink}>
                        {({isActive}) => (
                            <NavIcon
                                isActive={isActive}
                                component={"span"}
                                label={"AGB des Events"}
                            >
                                <IconSectionSign/>
                            </NavIcon>
                        )}
                    </NavLink>

                    <NavIcon
                        component={"a"}
                        href={`/util/qr?data=${encodeURIComponent(`${APP_URL}/events/${eventId}`)}`}
                        target={"_blank"}
                        label={"QR Code für Event"}
                    >
                        <IconQrcode/>
                    </NavIcon>

                    <NavIcon
                        component={"a"}
                        href={`/events/s/${eventId}`}
                        target={"_blank"}
                        label={"Öffentliche Event Seite"}
                    >
                        <IconExternalLink/>
                    </NavIcon>
                </Group>

                <Routes>
                    <Route index element={<Navigate to={"description"} replace/>}/>

                    <Route path={"description"} element={<EventDescription event={event}/>}/>

                    <Route path={"settings/*"} element={<EventSettingsRouter event={event}/>}/>

                    <Route path={"lists/*"} element={<EventListsRouter event={event}/>}/>

                    <Route path={"terms-and-conditions"} element={<EventAGB event={event}/>}/>
                </Routes>

                <Outlet/>
            </Grid.Col>
        </Grid>
    </>
}