import {EventModel} from "@/models/EventTypes.ts";
import {useForm} from "@mantine/form";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {Alert, Button, Checkbox, Divider, Group, Text, Title} from "@mantine/core";
import {useMutation} from "@tanstack/react-query";
import {queryClient} from "@/main.tsx";
import {IconSectionSign} from "@tabler/icons-react";
import TextEditor from "@/components/input/Editor";
import {useSettings} from "@/lib/settings.ts";
import {FormSchema} from "@/components/formUtil/formBuilder/types.ts";
import {showSuccessNotification} from "@/components/util.tsx";

/**
 * This component allows the user to edit the AGB of an event.
 * This includes the additional AGB for external events and the option to hide the event from the public.
 * @param event The event to edit the AGB for.
 */
export default function EditEventAGB({event}: { event: EventModel }) {

    const {pb} = usePB()

    const settings = useSettings()

    const stuveQuestions = JSON.parse(settings?.stuveEventQuestions?.value ?? `{ "fields":[] }`) as FormSchema

    const formValues = useForm({
        initialValues: {
            isStuveEvent: event.isStuveEvent ?? false,
            additionalAgb: event.externalAgb ?? "",
            hideFromPublic: event.hideFromPublic ?? false
        }
    })

    const editMutation = useMutation({
        mutationFn: async () => {

            // check if the event type has changed
            if (formValues.values.isStuveEvent != event.isStuveEvent) {
                // if the event is now a stuve event, we need to add the stuve default questions
                if (formValues.values.isStuveEvent) {
                    return await pb.collection("events").update(event.id, {
                        ...formValues.values,
                        defaultEntryQuestionSchema: {
                            fields: [
                                ...stuveQuestions.fields.map((field) => (
                                    {...field, meta: {...field.meta, required: true}}
                                )),
                                ...(event.defaultEntryQuestionSchema ?? {fields: []}).fields,
                            ]
                        }
                    })
                }

                // if the event is now an external event, we need to remove the stuve default questions we do this by filtering out the stuve questions (meta: required=true for this to work)
                return await pb.collection("events").update(event.id, {
                    ...formValues.values,
                    defaultEntryQuestionSchema: {
                        fields: (event.defaultEntryQuestionSchema ?? {fields: []}).fields
                            .filter((field) => (field.meta?.required !== true))
                    }
                })
            }


            // if the event type has not changed, we can just update the event
            return await pb.collection("events").update(event.id, {
                ...formValues.values,
            })
        },
        onSuccess: () => {
            showSuccessNotification("Gespeichert")
            return queryClient.invalidateQueries({queryKey: ["event", event.id]})
        }
    })

    return <form className="stack" onSubmit={formValues.onSubmit(() => editMutation.mutate())}>

        <Title order={4} c={"blue"}>AGB & Rechtliches</Title>

        <PocketBaseErrorAlert error={editMutation.error}/>

        {
            <Alert color={"blue"} title={
                formValues.values.isStuveEvent ? "StuVe Event" : "Externes Event"
            }>
                {
                    formValues.values.isStuveEvent ?
                        <>
                            StuVe Events (z.B. Uni-Party, Fachschafts-Event, ...) werden zusammen mit der
                            Studierenden Exekutive verwaltet. Es gelten die AGB der StuVe.
                        </> : <>
                            Externe Events (z.B. private Feiern, ...) haben eigene AGB
                            und die StuVe ist nicht der Veranstalter.

                            <Divider mt={"sm"} mb={"sm"}/>

                            <Checkbox
                                mt={"sm"}
                                label={"Im Kalender und in der Event-Liste verbergen. Alle mit Link können das Event trotzdem sehen."}
                                {...formValues.getInputProps("hideFromPublic", {type: "checkbox"})}
                            />
                        </>
                }
                <Checkbox
                    mt={"sm"}
                    label={"StuVe Event"}
                    {...formValues.getInputProps("isStuveEvent", {type: "checkbox"})}
                />
            </Alert>
        }

        <Alert color={"gray"} title={"Zusätzliche AGB"} icon={<IconSectionSign/>}>
            <Text size={"sm"} c={"dimmed"} mb={"sm"}>
                Für StuVe Events gelten immer auch die StuVe AGB. Für externe Events sollten eigene AGB angegeben
                werden.
            </Text>

            <TextEditor
                maxHeight={"300px"}
                noBorder
                fullToolbar
                value={formValues.values.additionalAgb}
                onChange={(value) => formValues.setFieldValue("additionalAgb", value)}
            />
        </Alert>

        {
            formValues.values.isStuveEvent != event.isStuveEvent && (
                <Alert color={"yellow"} title={"Achtung"}>
                    Du hast die Event-Art geändert. Dies verändert automatisch die Standard-Formularfelder für Listen.
                    <br/>
                    Bitte überprüfe die Listenfragen nach dem Speichern.
                </Alert>
            )
        }

        <Group>
            <Button
                type={"submit"}
                loading={editMutation.isPending}
                disabled={!formValues.isTouched()}
            >
                Speichern
            </Button>
        </Group>
    </form>
}