import {EventModel} from "@/models/EventTypes.ts";
import {hasLength, useForm} from "@mantine/form";
import dayjs from "dayjs";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {Button, Group, TextInput, Title} from "@mantine/core";
import {DateTimePicker} from "@mantine/dates";
import {useMutation} from "@tanstack/react-query";
import {queryClient} from "@/main.tsx";
import {showSuccessNotification} from "@/components/util.tsx";
import ShowHelp from "@/components/ShowHelp.tsx";

/**
 * This component allows the user to edit the data of an event.
 * Editable data includes the name, location, start and end date.
 * @param event The event to edit the data for.
 */
export default function EditEventData({event}: { event: EventModel }) {

    const {pb} = usePB()

    const formValues = useForm({
        initialValues: {
            name: event?.name ?? "",
            startDate: event?.startDate ? new Date(event.startDate) : null,
            endDate: event?.endDate ? new Date(event.endDate) : null,
            location: event?.location ?? "",
            description: event?.description ?? "",
        },
        validate: {
            name: hasLength({min: 4, max: 50}, 'Der Name muss zwischen 4 und 50 Zeichen lang sein.'),
            startDate: (value) => dayjs(value).isAfter(dayjs(), "day") ? null : "Das Startdatum muss in der Zukunft liegen.",
            endDate: (value, values) => dayjs(value).isAfter(dayjs(values.startDate), "day") ? null : "Das Enddatum muss nach dem Startdatum liegen.",
            location: hasLength({max: 500}, 'Der Ort darf maximal 500 Zeichen lang sein.'),
        }
    })

    const editMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("events").update(event.id, {
                ...formValues.values,
            })
        },
        onSuccess: () => {
            showSuccessNotification("Event gespeichert")
            return queryClient.invalidateQueries({queryKey: ["event", event.id]})
        }
    })

    return <form className="stack" onSubmit={formValues.onSubmit(() => editMutation.mutate())}>

        <Title order={4} c={"blue"}>Event Daten</Title>

        <ShowHelp>
            Hier kannst du die Daten für dein Event bearbeiten. Die Event-Admins können das Event bearbeiten.
            Alle Daten bis auf die Event-Admins sind öffentlich sichtbar.
        </ShowHelp>

        <PocketBaseErrorAlert error={editMutation.error}/>

        <TextInput
            label={"Name"}
            placeholder={"Name des Events"}
            required
            {...formValues.getInputProps("name")}
        />

        <TextInput
            label={"Ort"}
            placeholder={"z.B. 'Forum Uni Süd', 'Online', ..."}
            {...formValues.getInputProps("location")}
        />

        <DateTimePicker
            label={"Startdatum"}
            required
            {...formValues.getInputProps("startDate")}
        />

        <DateTimePicker
            label={"Enddatum"}
            required
            {...formValues.getInputProps("endDate")}
        />

        <Group>
            <Button
                type={"submit"}
                loading={editMutation.isPending}
                disabled={!formValues.isTouched()}
            >
                Speichern
            </Button>
        </Group>
    </form>
}