import {EventModel} from "@/models/EventTypes.ts";
import {useForm} from "@mantine/form";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {Button, Group, Title} from "@mantine/core";
import {useMutation} from "@tanstack/react-query";
import {queryClient} from "@/main.tsx";
import TextEditor from "@/components/input/Editor";
import {showSuccessNotification} from "@/components/util.tsx";
import ShowHelp from "@/components/ShowHelp.tsx";

/**
 * This component allows the user to edit the description of an event.
 * @param event The event to edit the description for.
 */
export default function EditEventDescription({event}: { event: EventModel }) {

    const {pb} = usePB()

    const formValues = useForm({
        initialValues: {
            description: event?.description ?? "",
        }
    })

    const editMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("events").update(event.id, {
                ...formValues.values,
            })
        },
        onSuccess: () => {
            showSuccessNotification("Beschreibung gespeichert")
            return queryClient.invalidateQueries({queryKey: ["event", event.id]})
        }
    })

    return <form className="stack" onSubmit={formValues.onSubmit(() => editMutation.mutate())}>

        <Title order={4} c={"blue"}>Event Beschreibung</Title>

        <ShowHelp>
            Hier kannst du eine Beschreibung für dein Event eintragen. Diese Beschreibung wird
            auch auf der öffentlichen Eventseite angezeigt.
        </ShowHelp>

        <PocketBaseErrorAlert error={editMutation.error}/>

        <TextEditor
            maxHeight={"500px"}
            fullToolbar
            placeholder={"Erzähle etwas über das Event..."}
            noBorder
            value={formValues.values.description}
            onChange={(value) => formValues.setFieldValue("description", value)}
            error={formValues.errors.description}
        />

        <Group>
            <Button
                type={"submit"}
                loading={editMutation.isPending}
                disabled={!formValues.isTouched()}
            >
                Speichern
            </Button>
        </Group>
    </form>
}