import {EventModel} from "@/models/EventTypes.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {useState} from "react";
import {useMutation} from "@tanstack/react-query";
import {queryClient} from "@/main.tsx";
import ImageSelect from "@/components/input/ImageSelect.tsx";
import {Avatar, Button, Group, Stack, Title} from "@mantine/core";
import {IconX} from "@tabler/icons-react";
import {showSuccessNotification} from "@/components/util.tsx";
import ShowHelp from "@/components/ShowHelp.tsx";

/**
 * This component allows the user to edit the image of an event.
 * @param event The event to edit the image for.
 */
export default function EditEventImage({event}: { event: EventModel }) {

    const {pb} = usePB()

    const [image, setImage] = useState<File | null>(null)

    const editMutation = useMutation({
        mutationFn: async () => {
            const formData = new FormData()
            formData.append("img", image as File)
            return await pb.collection("events").update(event.id, formData)
        },
        onSuccess: () => {
            showSuccessNotification("Bild gespeichert")
            setImage(null)
            return queryClient.invalidateQueries({queryKey: ["event", event.id]})
        }
    })

    return <>
        <Title order={4} c={"blue"}>Event Bild</Title>

        <ShowHelp>
            Setzte die Stimmung für dein Event mit einem Bild. Das Bild wird auf der öffentlichen
            Eventseite angezeigt.
            <br/>
            Am besten eignet sich ein <b>quadratisches</b> Bild ohne Text.
        </ShowHelp>

        <PocketBaseErrorAlert error={editMutation.error}/>


        {
            !image &&
            <div>
                <ImageSelect

                    onChange={(files) => setImage(files[0])}
                    fileCount={image ? 1 : 0}
                    maxFileCount={1}
                />
            </div>
        }

        {image && <Stack align={"center"} gap={"lg"}>

            <Avatar
                src={URL.createObjectURL(image)}
                alt={"Event Image"}
                color={"blue"}
                radius={20000}
                size="200px"
            />
            <Group>
                <Button
                    color={"red"}
                    leftSection={<IconX/>}
                    onClick={() => {
                        setImage(null)
                    }}
                >
                    Verwerfen
                </Button>

                <Button
                    onClick={() => editMutation.mutate()}
                    loading={editMutation.isPending}
                >
                    Bild speichern
                </Button>
            </Group>
        </Stack>
        }
    </>
}