import {EventLink, EventModel} from "@/models/EventTypes.ts";
import {hasLength, useForm} from "@mantine/form";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {ActionIcon, Alert, Button, Group, Stack, TextInput, Title, Tooltip} from "@mantine/core";
import {useMutation} from "@tanstack/react-query";
import {queryClient} from "@/main.tsx";
import {IconArrowBackUp, IconLink, IconPlus, IconTrash} from "@tabler/icons-react";
import {useListState} from "@mantine/hooks";
import {useEffect} from "react";
import ShowHelp from "@/components/ShowHelp.tsx";
import {showSuccessNotification} from "@/components/util.tsx";

/**
 * This component allows the user to edit the links of an event.
 * @param event The event to edit the links for.
 */
export default function EditEventLinks({event}: { event: EventModel }) {

    const {pb} = usePB()

    const [deletedLinks, deletedLinksHandlers] = useListState<EventLink>([]);

    const formValues = useForm({
        initialValues: {
            eventLinks: event.eventLinks ?? [] as EventLink[]
        },
        validate: {
            eventLinks: {
                name: hasLength({min: 4, max: 50}, 'Der Name des Links muss zwischen 4 und 50 Zeichen lang sein.'),
                url: hasLength({min: 4, max: 500}, 'Die URL des Links muss zwischen 4 und 500 Zeichen lang sein.'),
            }
        }
    })

    useEffect(() => {
        formValues.setValues({eventLinks: event.eventLinks ?? []})
        deletedLinksHandlers.setState([])
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [event])

    // This function marks a link for deletion
    const deleteLink = (index: number) => {
        const link = formValues.values.eventLinks[index]
        formValues.removeListItem("eventLinks", index)
        deletedLinksHandlers.append(link)
    }

    // This function puts a link back into the list of links
    const putBackLink = (index: number) => {
        const link = deletedLinks[index]
        formValues.insertListItem("eventLinks", link)
        deletedLinksHandlers.remove(index)
    }

    const editMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("events").update(event.id, {
                ...formValues.values,
            })
        },
        onSuccess: () => {
            showSuccessNotification("Links gespeichert")
            queryClient.invalidateQueries({queryKey: ["event", event.id]}).then(formValues.reset)
        }
    })

    return <form className="stack" onSubmit={formValues.onSubmit(() => editMutation.mutate())}>

        <Title order={4} c={"blue"}>Event Links</Title>

        <PocketBaseErrorAlert error={editMutation.error}/>

        {
            formValues.values.eventLinks.length === 0 &&
            <Alert color={"yellow"}>
                Für dieses Event sind keine Links hinterlegt.
            </Alert>
        }

        <ShowHelp>
            Hier kannst du Links zu deinem Event hinzufügen. Zum Beispiel zur Wiki-Seite, zum Instagram-Profil oder
            zur Ticket-Plattform. Klicke auf das Plus um einen neuen Link hinzuzufügen.
            Diese Links werden auch auf der öffentlichen Eventseite angezeigt.
        </ShowHelp>

        {formValues.values.eventLinks.map((_, index) => (
            <Group
                key={index}
                align={"top"}
                justify="center"
            >
                <TextInput
                    variant="unstyled"
                    style={{flex: 1}}
                    placeholder={"Link Name (z.B. Wiki--Seite, ...)"}
                    {...formValues.getInputProps(`eventLinks.${index}.name`)}
                />
                <TextInput
                    variant="unstyled"
                    rightSection={
                        <ActionIcon
                            aria-label={"delete link"}
                            variant={"transparent"}
                            color={"red"}
                            onClick={() => {
                                deleteLink(index)
                            }}
                        >
                            <IconTrash/>
                        </ActionIcon>
                    }
                    style={{flex: 3}}
                    placeholder={"https://wiki.stuve.uni-ulm.de"}
                    {...formValues.getInputProps(`eventLinks.${index}.url`)}
                />
            </Group>
        ))}

        {
            deletedLinks.length > 0 &&
            <Alert color={"red"} title={"Gelöschte Links"}>
                <Stack>
                    {
                        deletedLinks.map((link, index) => (
                            <Group key={index} align={"top"}>
                                <TextInput
                                    variant="unstyled"
                                    leftSection={<IconLink/>}
                                    style={{flex: 1}}
                                    placeholder={"Link Name"}
                                    value={link.name}
                                    disabled
                                />
                                <TextInput
                                    variant="unstyled"
                                    rightSection={
                                        <Tooltip label={"Link wiederherstellen"} position={"left"}>
                                            <ActionIcon
                                                aria-label={"put back link"}
                                                variant={"transparent"}
                                                color={"green"}
                                                onClick={() => putBackLink(index)}
                                            >
                                                <IconArrowBackUp/>
                                            </ActionIcon>
                                        </Tooltip>
                                    }
                                    style={{flex: 3}}
                                    placeholder={"https://example.com"}
                                    value={link.url}
                                    disabled
                                />
                            </Group>
                        ))
                    }
                </Stack>
            </Alert>
        }

        <Group>
            <Button
                variant={"light"}
                onClick={() => {
                    formValues.insertListItem("eventLinks", {
                        name: "",
                        url: ""
                    })
                }}
                leftSection={<IconPlus/>}
            >
                Link
            </Button>

            <Button
                type={"submit"}
                loading={editMutation.isPending}
            >
                Speichern
            </Button>
        </Group>
    </form>
}