import {EventModel} from "@/models/EventTypes.ts";
import {useForm} from "@mantine/form";
import {UserModal} from "@/models/AuthTypes.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {Button, Group, Title} from "@mantine/core";
import UserInput from "@/components/auth/UserInput.tsx";
import {useMutation} from "@tanstack/react-query";
import {queryClient} from "@/main.tsx";
import {showSuccessNotification} from "@/components/util.tsx";
import ShowHelp from "@/components/ShowHelp.tsx";

/**
 * This component allows the user to edit the admins of the event and the event lists.
 * @param event The event to edit the data for.
 */
export default function EditEventMembers({event}: { event: EventModel }) {

    const {user, pb} = usePB()

    const formValues = useForm({
        initialValues: {
            eventAdmins: event?.expand?.eventAdmins ?? [user] as UserModal[],
            eventListAdmins: event?.expand?.eventListAdmins ?? [] as UserModal[],
        },
        validate: {
            eventAdmins: (value) => value.length > 0 ? null : "Es muss mindestens ein Admin ausgewählt werden.",
            eventListAdmins: (value, values) =>
                value.filter((admin) => values.eventAdmins.map((admin) => admin.id).includes(admin.id)).length === 0
                    ? null
                    : "Ein Event Admin kann nicht gleichzeitig Listen Admin sein."
        }
    })

    const editMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("events").update(event.id, {
                eventAdmins: [...formValues.values.eventAdmins.map((member) => member.id), user?.id],
                eventListAdmins: formValues.values.eventListAdmins.map((member) => member.id)
            })
        },
        onSuccess: () => {
            showSuccessNotification("Event und Listen Admins gespeichert")
            return queryClient.invalidateQueries({queryKey: ["event", event.id]})
        }
    })

    return <form className="stack" onSubmit={formValues.onSubmit(() => editMutation.mutate())}>

        <Title order={4} c={"blue"}>Event Admins</Title>

        <ShowHelp>
            <Title order={6}>Event Admins</Title>
            Event Admin können alle Einstellungen des Events bearbeiten.
            Sie können auch alle Listen bearbeiten und verwalten.
            <br/>
            <br/>

            <Title order={6}>Listen Admins</Title>
            Listen Admin können Listen bearbeiten und verwalten.
            Sie können <b>keine</b> Einstellungen des Events bearbeiten.
            <br/>
            <br/>
            Eine Person kann nicht gleichzeitig Event Admin und Listen Admin sein.
        </ShowHelp>

        <PocketBaseErrorAlert error={editMutation.error}/>


        <UserInput
            required
            label={"Event Admins"}
            description={"Die Event-Admins können das ganze Event bearbeiten."}
            error={formValues.errors.eventAdmins}
            selectedRecords={formValues.values.eventAdmins}
            setSelectedRecords={(records) => formValues.setFieldValue("eventAdmins", records)}
        />

        <UserInput
            label={"Listen Admins"}
            description={"Die Listen Admins können Listen bearbeiten und verwalten."}
            error={formValues.errors.eventListAdmins}
            selectedRecords={formValues.values.eventListAdmins}
            setSelectedRecords={(records) => formValues.setFieldValue("eventListAdmins", records)}
        />

        <Group>
            <Button
                type={"submit"}
                loading={editMutation.isPending}
                disabled={!formValues.isTouched()}
            >
                Speichern
            </Button>
        </Group>
    </form>
}