import {EventModel} from "@/models/EventTypes.ts";
import {Alert, Anchor, Menu} from "@mantine/core";
import {
    IconGhost2,
    IconInfoCircle,
    IconLink,
    IconPencil,
    IconPhotoUp,
    IconSectionSign,
    IconUserEdit
} from "@tabler/icons-react";
import EditEventImage from "./EditEventImage.tsx";
import EditEventData from "./EditEventData.tsx";
import EditEventAGB from "./EditEventAGB.tsx";
import EditEventDescription from "./EditEventDescription.tsx";
import ArchiveEvent from "./ArchiveEvent.tsx";
import EditEventLinks from "./EditEventLinks.tsx";
import {Navigate, NavLink, Outlet, Route, Routes} from "react-router-dom";
import {ReactNode} from "react";
import EditEventMembers from "@/pages/events/e/:eventId/EventComponents/EventSettings/EditEventMembers.tsx";
import {useEventRights} from "@/pages/events/util.ts";

const nav = [
    {
        label: "Allgemein",
        children: [
            {
                title: "Daten",
                icon: <IconPencil size={16}/>,
                to: "data"
            },
            {
                title: "Bild",
                icon: <IconPhotoUp size={16}/>,
                to: "image"
            },
            {
                title: "Beschreibung",
                icon: <IconInfoCircle size={16}/>,
                to: "description"
            }
        ]
    },
    {
        label: "Sonstiges",
        children: [
            {
                title: "Rechtliches",
                icon: <IconSectionSign size={16}/>,
                to: "agb"
            },
            {
                title: "Links",
                icon: <IconLink size={16}/>,
                to: "links"
            }
        ]
    },
    {
        label: "Danger Zone",
        children: [
            {
                title: "Admins",
                icon: <IconUserEdit size={16}/>,
                to: "admins"
            },
            {
                title: "Archivieren",
                icon: <IconGhost2 size={16}/>,
                to: "archive"
            }
        ]
    }
]

/**
 * Dropdown menu to navigate between different settings of an event
 * @param event - the event to edit
 * @param target - the trigger element for the dropdown
 */
export const EventSettingsMenu = ({event, target}: { event: EventModel, target: ReactNode }) => {
    return <>
        <Menu
            withArrow shadow="md" width={200} trigger="click-hover" loop={false}
            withinPortal={false} trapFocus={false} menuItemTabIndex={0}
        >
            <Menu.Target>
                {target}
            </Menu.Target>

            <Menu.Dropdown>
                {
                    nav.map(({label, children,}, index, array) => <div key={index}>
                        <Menu.Label>{label}</Menu.Label>
                        {children.map(({title, icon, to}) => (
                            <Anchor component={NavLink} to={`/events/e/${event.id}/settings/${to}`} key={to}>
                                {({isActive}) => (
                                    <Menu.Item color={isActive ? "blue" : undefined} leftSection={icon}>
                                        {title}
                                    </Menu.Item>
                                )}
                            </Anchor>
                        ))}
                        {index !== array.length - 1 && <Menu.Divider/>}
                    </div>)
                }
            </Menu.Dropdown>
        </Menu>
    </>
}

/**
 * Router to display the different settings of an event, depending on the current route
 * Displays an error message if the user is not an admin of the event
 * @param event - the event to edit
 */
export default function EventSettingsRouter({event}: { event: EventModel }) {

    const {canEditEvent} = useEventRights(event)

    if (!canEditEvent) {
        return (
            <Alert color={"red"}>
                Nur Admins können die Event Einstellungen bearbeiten.
            </Alert>
        )
    }

    return <>
        <Routes>
            <Route index element={<Navigate to={"data"} replace/>}/>
            <Route path="data" element={<EditEventData event={event}/>}/>
            <Route path="image" element={<EditEventImage event={event}/>}/>
            <Route path="description" element={<EditEventDescription event={event}/>}/>
            <Route path="agb" element={<EditEventAGB event={event}/>}/>
            <Route path="links" element={<EditEventLinks event={event}/>}/>
            <Route path="admins" element={<EditEventMembers event={event}/>}/>
            <Route path="archive" element={<ArchiveEvent event={event}/>}/>
        </Routes>
        <Outlet/>
    </>
}