import {useQuery} from "@tanstack/react-query";
import {Alert, Breadcrumbs, Button, Group, LoadingOverlay, Title} from "@mantine/core";
import {
    IconCheckupList,
    IconClockCog,
    IconForms,
    IconLock,
    IconLockOpen,
    IconSettings,
    IconUserCog
} from "@tabler/icons-react";
import {Link, Navigate, NavLink, Route, Routes, useParams} from "react-router-dom";
import InnerHtml from "@/components/InnerHtml";
import {EventModel} from "@/models/EventTypes.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import SlotsTable from "./EventListSlotsTable";
import EventListSettings from "@/pages/events/e/:eventId/EventLists/:listId/EventListSettings/EventListSettings.tsx";
import EventListEntryQuestionSettings
    from "@/pages/events/e/:eventId/EventLists/:listId/EventListSettings/EventListEntryQuestionSettings.tsx";
import EventListEntryStatusSettings
    from "@/pages/events/e/:eventId/EventLists/:listId/EventListSettings/EventListEntryStatusSettings.tsx";
import TextWithIcon from "@/components/layout/TextWithIcon";


export default function EventListRouter({event}: { event: EventModel }) {

    const {pb} = usePB()

    const {listId} = useParams() as { listId: string }

    const listQuery = useQuery({
        queryKey: ["event", event.id, "list", listId],
        queryFn: async () => (await pb.collection("eventLists").getOne(listId))
    })

    if (listQuery.isLoading) {
        return <LoadingOverlay/>
    }

    if (!listQuery.data || listQuery.isError) {
        return <PocketBaseErrorAlert error={listQuery.error}/>
    }

    const list = listQuery.data

    return <div className={"stack"}>
        <Breadcrumbs>
            <Link to={`/events/e/${event.id}/lists/overview`}>
                <Title order={2}>
                    Event Listen
                </Title>
            </Link>
            <Link to={`/events/e/${event.id}/lists/overview/${list.id}`}>
                <Title order={2}>
                    {list.name}
                </Title>
            </Link>
        </Breadcrumbs>

        <Alert color={list.open ? "green" : "red"}>
            <TextWithIcon icon={list.open ? <IconLockOpen size={16}/> : <IconLock size={16}/>}>
                Liste ist für Anmeldungen <b>{list.open ? "geöffnet" : "geschlossen"}</b>
            </TextWithIcon>
            <br/>
            <TextWithIcon icon={<IconUserCog size={16}/>}>
                Anmeldung für Personen mit {list.onlyStuVeAccounts ?
                <><b>StuVe</b> Account</> : <>StuVe <b>und</b> Gast Account</>}
            </TextWithIcon>
            <br/>
            <TextWithIcon icon={<IconClockCog size={16}/>}>
                Überlappende Einträge sind {!list.allowOverlappingEntries && <b>nicht</b>} erlaubt
            </TextWithIcon>
        </Alert>

        <div className={"section"}>
            {list.description ? <>
                    <Title order={4}>
                        Beschreibung
                    </Title>
                    <InnerHtml html={list.description}/>
                </>
                :
                <Alert color={"orange"}>
                    Keine Beschreibung vorhanden
                </Alert>
            }
        </div>

        <Group>
            {[
                {
                    icon: <IconClockCog/>,
                    to: `/events/e/${event.id}/lists/overview/${list.id}/entries`,
                    title: "Zeitslots"
                },
                {
                    icon: <IconSettings/>,
                    to: `/events/e/${event.id}/lists/overview/${list.id}/settings`,
                    title: "Einstellungen"
                },
                {
                    icon: <IconForms/>,
                    to: `/events/e/${event.id}/lists/overview/${list.id}/questions`,
                    title: "Formular"
                },
                {
                    icon: <IconCheckupList/>,
                    to: `/events/e/${event.id}/lists/overview/${list.id}/status`,
                    title: "Eintrag Status"
                }
            ].map(({icon, to, title}) => (
                <NavLink to={to} key={title} end>
                    {({isActive}) =>
                        <Button
                            variant={isActive ? "light" : "transparent"}
                            leftSection={icon}
                        >
                            {title}
                        </Button>
                    }
                </NavLink>
            ))}
        </Group>

        <Routes>
            <Route index element={<Navigate to={"entries"} replace/>}/>
            <Route path={"entries"} element={<SlotsTable event={event} list={list}/>}/>
            <Route path={"settings"} element={<EventListSettings list={list} event={event}/>}/>
            <Route path={"questions"} element={<EventListEntryQuestionSettings list={list} event={event}/>}/>
            <Route path={"status"} element={<EventListEntryStatusSettings list={list} event={event}/>}/>
        </Routes>
    </div>
}