import {EventListModel, EventModel} from "@/models/EventTypes.ts";
import {useForm} from "@mantine/form";
import {ActionIcon, Button, Group, TextInput, Title, Tooltip} from "@mantine/core";
import TextEditor from "@/components/input/Editor";
import {useMutation} from "@tanstack/react-query";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {queryClient} from "@/main.tsx";
import {showSuccessNotification} from "@/components/util.tsx";
import {IconStarFilled, IconStarOff, IconTrash} from "@tabler/icons-react";
import {useNavigate} from "react-router-dom";
import {useConfirmModal} from "@/components/ConfirmModal.tsx";
import {CheckboxCard} from "@/components/input/CheckboxCard";

export default function EventListSettings({list, event}: { list: EventListModel, event: EventModel }) {
    const {pb} = usePB()
    const navigate = useNavigate()

    const formValues = useForm({
        initialValues: {
            name: list.name,
            open: list.open,
            allowOverlappingEntries: list.allowOverlappingEntries,
            onlyStuVeAccounts: list.onlyStuVeAccounts,
            favorite: list.favorite,
            description: list.description || ""
        }
    })

    const {ConfirmModal, toggleConfirmModal} = useConfirmModal({
        onConfirm: () => deleteMutation.mutate(),
        description: "Bist du sicher, dass du diese Liste löschen möchtest? " +
            "Alle Anmeldungen werden ebenfalls gelöscht. " +
            "Diese Aktion kann nicht rückgängig gemacht werden.",
    })

    const editMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("eventLists").update(list.id, {
                ...formValues.values
            })
        },
        onSuccess: () => {
            showSuccessNotification("Liste gespeichert")
            queryClient.invalidateQueries({queryKey: ["event", event.id, "list", list.id]})
            queryClient.invalidateQueries({queryKey: ["event", event.id, "favorites"]})
        }
    })

    const deleteMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("eventLists").delete(list.id)
        },
        onSuccess: () => {
            showSuccessNotification("Liste gelöscht")
            queryClient.invalidateQueries({queryKey: ["event", event.id]})
            navigate(`/events/e/${event.id}/lists/overview`)
        }
    })

    return <form
        className={"section stack"}
        onSubmit={formValues.onSubmit(() => editMutation.mutate())}
    >
        <Title order={4} c={"blue"}>Listeneinstellungen</Title>

        <PocketBaseErrorAlert error={editMutation.error}/>

        <ConfirmModal/>

        <TextInput
            label={"Name"}
            placeholder={"Name der Liste"}
            required
            rightSection={<Tooltip label={"Liste im favoriten Menu anzeigen"}>
                <ActionIcon
                    color={"blue"}
                    onClick={() => formValues.setFieldValue("favorite", !formValues.values.favorite)}
                    variant={"transparent"}
                >
                    {
                        formValues.values.favorite ? <IconStarFilled/> : <IconStarOff/>
                    }
                </ActionIcon>
            </Tooltip>}
            {...formValues.getInputProps("name")}
        />

        <CheckboxCard
            label={"Anmeldungen erlauben"}
            description={"Ob sich Personen für Zeitslots dieser Liste anmelden können"}
            {...formValues.getInputProps("open", {type: "checkbox"})}
        />

        <CheckboxCard
            label={"Nur StuVe Accounts"}
            description={"Nur Personen mit StuVe IT Account können sich für diese Liste Anmelden"}
            {...formValues.getInputProps("onlyStuVeAccounts", {type: "checkbox"})}
        />

        <CheckboxCard
            label={"Überlappende Anmeldungen erlauben"}
            description={" Wenn diese Option aktiviert ist, kann sich eine Person in überlappende Zeitslots anmelden. " +
                "Dabei werden alle Zeitslots von diesem Event beachtet. " +
                "Bei dem Verschieden von Anmeldungen wirds diese Regel nicht beachtet!"}
            {...formValues.getInputProps("allowOverlappingEntries", {type: "checkbox"})}
        />

        <TextEditor
            maxHeight={"500px"}
            fullToolbar
            value={formValues.values.description}
            onChange={(value) => formValues.setFieldValue("description", value)}
        />

        <Group>

            <Button
                variant={"outline"}
                color={"red"}
                onClick={toggleConfirmModal}
                leftSection={<IconTrash/>}
            >
                Liste löschen
            </Button>

            <Button
                type={"submit"}
                loading={false}
                disabled={!formValues.isTouched()}
            >
                Speichern
            </Button>
        </Group>
    </form>
}