import {EventListModel, EventListSlotsWithEntriesCountModel, EventModel} from "@/models/EventTypes.ts";
import {useQuery} from "@tanstack/react-query";
import {useState} from "react";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import classes from "./EventListSlotEntriesTable.module.css";
import {Loader, Pagination, Stack, Text, ThemeIcon} from "@mantine/core";
import {transformData} from "@/components/formUtil/formTable";
import {EventListSlotEntryRow} from "./EventListSlotEntryRow.tsx";
import {IconDatabaseOff} from "@tabler/icons-react";

export const EventListSlotEntriesTable = ({slot, list, event, refetch, visible}:
                                              {
                                                  visible: boolean,
                                                  refetch: () => void,
                                                  slot: EventListSlotsWithEntriesCountModel,
                                                  list: EventListModel,
                                                  event: EventModel
                                              }) => {
    const {pb} = usePB()

    const [page, setPage] = useState(1)

    const query = useQuery({
        queryKey: ["event", event.id, "list", list.id, "slot", slot.id, "entriesWithUser", page],
        queryFn: async () => {

            const res = await pb.collection("eventListSlotEntriesWithUser").getList(page, 50, {
                filter: `eventListsSlot='${slot.id}'`,
                expand: "user"
            })

            return {
                ...res,
                items: res.items.map(item => ({
                    ...item,
                    questionData: transformData(item.questionData ?? {}),
                    statusData: transformData(item.statusData ?? {}),
                }))
            }
        },
        enabled: visible
    })

    if (query.isLoading) {
        return <Stack align={"center"} gap={"xs"}>
            <Loader size={"sm"}/>
            <Text size={"xs"} c={"dimmed"}>Anmeldungen werden geladen</Text>
        </Stack>
    }

    if (query.data?.totalItems === 0) {
        return <Stack align={"center"} gap={"xs"}>
            <ThemeIcon variant={"transparent"} color={"gray"} size={"md"}>
                <IconDatabaseOff/>
            </ThemeIcon>
            <Text size={"xs"} c={"dimmed"}>Keine Anmeldungen vorhanden</Text>
        </Stack>
    }

    if (query.isError) return <PocketBaseErrorAlert error={query.error}/>

    return <div className={classes.table}>
        {
            query.data?.items.map(entry => (
                <EventListSlotEntryRow
                    key={entry.id}
                    entry={entry}
                    slot={slot}
                    list={list}
                    event={event}
                    refetch={() => {
                        query.refetch()
                        refetch()
                    }}
                />
            ))
        }

        <div className={classes.bottomRow}>
            <Text size={"xs"} c={"dimmed"}>
                {query.data?.totalItems} Anmeldungen
            </Text>
            <Pagination size={"xs"} value={page} onChange={setPage} total={query.data?.totalPages ?? 1}/>
        </div>
    </div>
}