import {EventListModel, EventModel} from "@/models/EventTypes.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {useQuery} from "@tanstack/react-query";
import {Box, Button, Center, Pagination, Title} from "@mantine/core";
import {IconClockPlus} from "@tabler/icons-react";
import {useState} from "react";
import {useDisclosure} from "@mantine/hooks";
import classes from './index.module.css'
import {EventListSlotRow} from "./EventListSlotRow.tsx";
import UpsertEventListSlot from "@/pages/events/e/:eventId/EventLists/components/UpsertEventListSlot.tsx";

/**
 * Renders a table with all slots for a list
 * Lets the user add new slots, edit existing slots and view the entries for each slot
 * @param list - list to render slots for
 * @param event - event the list belongs to
 * @constructor
 */
export default function SlotsTable({list, event}: { event: EventModel, list: EventListModel }) {

    const {pb} = usePB()

    const [page, setPage] = useState(1)

    const [showNewSlotForm, showNewSlotFormHandler] = useDisclosure(false)

    const query = useQuery({
        queryKey: ["eventListSlots", {event: list.event}, page],
        queryFn: async () => (
            await pb.collection("eventListSlotsWithEntriesCount").getList(page, 50, {
                filter: `eventList='${list.id}'`,
                sort: "startDate"
            })
        )
    })

    return <>
        <PocketBaseErrorAlert error={query.error}/>

        <div className={classes.table}>
            {query.data?.items.map(slot => (
                <EventListSlotRow
                    key={slot.id} event={event} list={list} slot={slot}
                    refetch={query.refetch}
                />
            ))}
        </div>

        <Box>
            {
                showNewSlotForm ? <div className={"section-transparent stack"}>
                    <Title order={4} c={"blue"}>
                        Neuen Slot hinzufügen
                    </Title>

                    <UpsertEventListSlot list={list} onSuccess={() => {
                        showNewSlotFormHandler.close()
                        query.refetch()
                    }} onAbort={showNewSlotFormHandler.close}/>
                </div> : <Center>
                    <Button
                        onClick={showNewSlotFormHandler.toggle}
                        variant={"light"}
                        size={"xs"}
                        radius={"xl"}
                        leftSection={<IconClockPlus/>}
                    >
                        Neuen Slot hinzufügen
                    </Button>
                </Center>
            }
        </Box>
        <Center>
            <Pagination size={"xs"} value={page} onChange={setPage} total={query.data?.totalPages ?? 1}/>
        </Center>
    </>
}