import {EventListModel, EventModel} from "@/models/EventTypes.ts";
import {
    ActionIcon,
    Autocomplete,
    Center,
    Collapse,
    Group,
    Loader,
    Pagination,
    Text,
    Title,
    Tooltip
} from "@mantine/core";
import {IconFilter, IconFilterOff, IconUserSearch} from "@tabler/icons-react";
import {useQuery} from "@tanstack/react-query";
import {useDebouncedValue, useDisclosure} from "@mantine/hooks";
import {usePB} from "@/lib/pocketbase.tsx";
import {useState} from "react";
import {onlyUnique} from "@/lib/util.ts";
import EventListSearchResult from "@/pages/events/e/:eventId/EventLists/EventListSearch/EventListSearchResult.tsx";
import ListSelect from "@/pages/events/e/:eventId/EventLists/ListSelect.tsx";


export default function ListSearch({event}: { event: EventModel }) {

    const {pb} = usePB()

    const [showFilter, showFilterHandler] = useDisclosure(false)

    const [searchQueryString, setSearchQueryString] = useState('')
    const [selectedLists, setSelectedLists] = useState<EventListModel[]>([])

    const [debouncedSearchQueryString] = useDebouncedValue(searchQueryString, 200)

    const [page, setPage] = useState(1)

    const searchQuery = useQuery({
        queryKey: ["eventListSearch", {event: event.id}, {debouncedSearchQueryString}, {page}, {selectedLists}],
        queryFn: async () => {

            const filter: string[] = [`event='${event.id}'`]

            if (debouncedSearchQueryString) {
                filter.push(`user.username ~ '${debouncedSearchQueryString.trim().replace(" ", ".")}'`)
            }

            if (selectedLists.length > 0) {
                filter.push(`(${selectedLists.map(l => `eventList='${l.id}'`).join(" || ")})`)
            }

            return await pb.collection("eventListSlotEntriesWithUser").getList(1, 50, {
                filter: filter.join(" && "),
                expand: "user"
            })
        }
    })

    return <div className={"stack"}>
        <Title order={4} c={"blue"}>Listen Durchsuchen</Title>

        <Autocomplete
            placeholder={"Nach Namen suchen ..."}
            leftSection={<IconUserSearch/>}
            rightSection={searchQuery.isLoading ? <Loader size={"xs"}/> : (
                <Tooltip label={"Anmeldungen Filtern"} withArrow>
                    <ActionIcon
                        onClick={showFilterHandler.toggle}
                        variant={"transparent"}
                        aria-label={"toggle filter"}
                    >
                        {showFilter ? <IconFilterOff/> : <IconFilter/>}
                    </ActionIcon>
                </Tooltip>
            )}
            data={
                searchQuery
                    .data
                    ?.items
                    .map(e => e.id)
                    .filter(onlyUnique) ?? []
            }
            value={searchQueryString} onChange={setSearchQueryString}
        />

        <Collapse in={showFilter}>
            <ListSelect
                event={event}
                selectedRecords={selectedLists}
                setSelectedRecords={setSelectedLists}
                placeholder={"Anmeldungen nur für bestimmte Listen anzeigen"}
            />
        </Collapse>

        <Group justify={"space-between"}>
            <Text c={"dimmed"} size={"xs"}>
                {searchQueryString ? `Suche nach Person '${searchQueryString}'` : "Alle Anmeldungen für dieses Event"}
            </Text>

            <Text c={"dimmed"} size={"xs"}>
                {searchQuery.data?.totalItems ?? 0} {searchQueryString ? "Ergebnisse" : "Anmeldungen"}
            </Text>
        </Group>


        {/*
        todo: more filter
        <div className={"section stack"}>
            <Title order={4} c={"blue"}>
                Filter
            </Title>
            <Text> Listenauswahl </Text>
            <Text> Formularfelder Auswahl </Text>
            <Text> Statusfelder Auswahl </Text>
        </div>
        */}

        {
            searchQuery.data?.items.map((entry, index) => (
                <EventListSearchResult
                    key={index}
                    entry={entry}
                    refetch={() => searchQuery.refetch()}
                />
            ))
        }

        <Center>
            <Pagination size={"xs"} total={searchQuery.data?.totalPages ?? 0} value={page} onChange={setPage}/>
        </Center>
    </div>
}