import {EventModel} from "@/models/EventTypes.ts";
import {Alert, Loader, Title} from "@mantine/core";
import {useMutation} from "@tanstack/react-query";
import {queryClient} from "@/main.tsx";
import {FormSchema} from "@/components/formUtil/formBuilder/types.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import FormBuilder from "@/components/formUtil/formBuilder";
import ShowHelp from "@/components/ShowHelp.tsx";
import {useSettings} from "@/lib/settings.ts";
import {showSuccessNotification} from "@/components/util.tsx";
import {IconAlertTriangle, IconInfoCircle} from "@tabler/icons-react";

/**
 * This component allows the user to edit the default questions for all eventLists of an event.
 * @param event The event to edit the default questions for.
 */
export default function EditEventDefaultEntryQuestionSchema({event}: { event: EventModel }) {

    const {pb} = usePB()

    const settings = useSettings()

    const editMutation = useMutation({
        mutationFn: async (values: FormSchema) => {
            return await pb.collection("events").update(event.id, {
                defaultEntryQuestionSchema: values
            })
        },
        onSuccess: () => {
            showSuccessNotification("Fragen gespeichert")
            return queryClient.invalidateQueries({queryKey: ["event", event.id]})
        }
    })

    if (!settings) return <div className="stack">
        <Title order={4} c={"blue"}>Formularfelder für Listen</Title>
        <Loader/>
    </div>

    return <div className="stack">
        <Title order={4} c={"blue"}>Formularfelder für Listen</Title>

        <ShowHelp>
            <Title order={6}>Was sind Formularfelder?</Title>
            Formularfelder sind die Felder, die eine Person ausfüllen muss,
            um sich für einen Zeitslot in einer Liste anzumelden.
            <br/>
            <br/>

            <Title order={6}>Standard-Formularfelder</Title>

            Die hier aufgeführten Formularfelder werden standardmäßig für alle Listen dieses Events verwendet.
            Bitte beachte, dass Änderungen an den Standard-Formularfeldern sofort wirksam werden.
            Sobald du Felder hinzufügst oder anpasst, werden diese Änderungen automatisch in allen Listen,
            einschließlich der bereits bestehenden, übernommen.
            In den Einstellungen der einzelnen Listen können zusätzliche Felder hinzugefügt werden.

            {
                event.isStuveEvent && <>
                    <br/>
                    <br/>
                    <Title order={6}>StuVe-Formularfelder</Title>
                    Da dein Event ein StuVe Event ist, werden <b>zusätzlich</b> die StuVe-Formularfelder
                    (Datenschutz, ...) verwendet.
                </>
            }
        </ShowHelp>

        <Alert color={"orange"} icon={<IconAlertTriangle/>}>
            Wenn du Felder entfernst, werden alle Daten,
            die für diese Felder gespeichert wurden nicht mehr angezeigt.
        </Alert>

        <Alert color={"green"} icon={<IconInfoCircle/>}>
            <b>(Vor-) Name</b> und <b>Email</b> werden automatisch bei der Anmeldung über den User-Account erfasst
            und sollten daher nicht zusätzlich in Formularfeldern abgefragt werden.
        </Alert>

        <PocketBaseErrorAlert error={editMutation.error}/>

        <FormBuilder
            withPreview
            defaultValue={event.defaultEntryQuestionSchema ?? {fields: []}}
            onSubmit={(values) => editMutation.mutate(values)}
        />
    </div>
}