import {EventModel} from "@/models/EventTypes.ts";
import {Alert, Loader, Title} from "@mantine/core";
import {useMutation} from "@tanstack/react-query";
import {queryClient} from "@/main.tsx";
import {FormSchema} from "@/components/formUtil/formBuilder/types.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import FormBuilder from "@/components/formUtil/formBuilder";
import ShowHelp from "@/components/ShowHelp.tsx";
import {useSettings} from "@/lib/settings.ts";
import {showSuccessNotification} from "@/components/util.tsx";
import {IconAlertTriangle} from "@tabler/icons-react";

/**
 * This component allows the user to edit the default entry status fields for all eventLists of an event.
 * @param event The event to edit the default entry status fields for.
 */
export default function EditEventDefaultEntryStatusSchema({event}: { event: EventModel }) {

    const {pb} = usePB()

    const settings = useSettings()

    const editMutation = useMutation({
        mutationFn: async (values: FormSchema) => {
            return await pb.collection("events").update(event.id, {
                defaultEntryStatusSchema: values
            })
        },
        onSuccess: () => {
            showSuccessNotification("Status Felder gespeichert")
            return queryClient.invalidateQueries({queryKey: ["event", event.id]})
        }
    })

    if (!settings) return <div className="stack">
        <Title order={4} c={"blue"}>Status für Listen</Title>
        <Loader/>
    </div>

    return <div className="stack">
        <Title order={4} c={"blue"}>Eintrag-Status für Listen</Title>

        <ShowHelp>
            <Title order={6}>Was sind Eintrag-Status-Felder?</Title>
            In diesen Feldern wird der Status eines Eintrags in einer Liste gespeichert.
            Diese Felder können nur von den Listen Admins bearbeitet werden.
            Mit Hilfe des Status kann z.B. gespeichert werden, ob eine Person die sich eingetragen hat
            erschiene ist oder nicht.

            <br/>
            <br/>

            <Title order={6}>Standard-Status-Felder</Title>
            Die hier aufgeführten Status-Felder werden standardmäßig für alle Listen verwendet.
            Bitte beachte, dass Änderungen an den Standard-Status-Felder sofort wirksam werden.
            Sobald du Felder hinzufügst oder anpasst, werden diese Änderungen automatisch in allen Listen,
            einschließlich der bereits bestehenden, übernommen.
            In den Einstellungen der einzelnen Listen können zusätzliche Felder hinzugefügt werden.
        </ShowHelp>

        <Alert color={"orange"} icon={<IconAlertTriangle/>}>
            Wenn du Felder entfernst, werden alle Daten,
            die für diese Felder gespeichert wurden nicht mehr angezeigt.
        </Alert>

        <PocketBaseErrorAlert error={editMutation.error}/>

        <FormBuilder
            withPreview
            defaultValue={event.defaultEntryStatusSchema ?? {fields: []}}
            onSubmit={(values) => editMutation.mutate(values)}
        />
    </div>
}