import {EventListModel, EventModel} from "@/models/EventTypes.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {useMutation, useQuery} from "@tanstack/react-query";
import {
    ActionIcon,
    Alert,
    Autocomplete,
    Box,
    Button,
    Center,
    Collapse,
    Group,
    Loader,
    Pagination,
    Text,
    TextInput,
    ThemeIcon,
    Title,
    Tooltip,
    UnstyledButton
} from "@mantine/core";
import {hasLength, useForm} from "@mantine/form";
import TextEditor from "@/components/input/Editor";
import {Link, useNavigate} from "react-router-dom";
import {IconArrowRight, IconListSearch, IconLock, IconLockOpen, IconPlus} from "@tabler/icons-react";
import {useRef, useState} from "react";
import {useDebouncedState} from "@mantine/hooks";
import {onlyUnique} from "@/lib/util.ts";

/**
 * Renders a single List as clickable row (link)
 */
const ListRow = ({list, event}: { list: EventListModel, event: EventModel }) => {
    return (
        <UnstyledButton
            component={Link}
            to={`/events/e/${event.id}/lists/overview/${list.id}`}
            className={"section hover"}
        >
            <Group w={"calc(100%)"} justify="space-between" wrap={"nowrap"}>

                <Text truncate={"end"}>
                    {list.name}
                </Text>

                <Group wrap={"nowrap"}>
                    <Tooltip
                        label={list.open ? "Anmeldungen aktiviert" : "Anmeldungen deaktiviert"}
                        color={list.open ? "green" : "red"} withArrow
                    >
                        <ThemeIcon
                            aria-label={"edit list"}
                            variant={"transparent"}
                            color={list.open ? "green" : "red"}
                        >
                            {list.open ? <IconLockOpen/> : <IconLock/>}
                        </ThemeIcon>
                    </Tooltip>


                    <ThemeIcon
                        aria-label={"open list"}
                        variant={"transparent"}
                        color={"blue"}
                    >
                        <IconArrowRight/>
                    </ThemeIcon>
                </Group>
            </Group>
        </UnstyledButton>
    )
}

/**
 * ListsOverview Component
 * Renders a list of all lists for a given event
 * Allows to create new lists
 * @param event - the event for which the lists should be displayed
 */
export default function EventListsOverview({event}: { event: EventModel }) {

    const {pb} = usePB()

    const [activePage, setPage] = useState(1)

    const [listSearchQuery, setListSearchQuery] = useDebouncedState('', 200)

    const navigate = useNavigate()

    const formValues = useForm({
        initialValues: {
            name: "",
            description: "",
        },
        validate: {
            name: hasLength({min: 4, max: 50}, 'Der Name der Liste muss zwischen 4 und 50 Zeichen lang sein.')
        }
    })

    const listsQuery = useQuery({
        queryKey: ["event", event.id, "lists", `page-${activePage}`, `search-${listSearchQuery}`],
        queryFn: async () => (await pb.collection('eventLists').getList(1, 10, {
            filter: `event = '${event.id}'` + (listSearchQuery ? ` && name ~ '${listSearchQuery}'` : ''),
        }))
    })

    const createListMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection('eventLists').create({
                event: event.id,
                name: formValues.values.name,
                description: formValues.values.description,
                entryQuestionSchema: {fields: []},
                open: false
            })
        },
        onSuccess: (list) => {
            listsQuery.refetch().then(() => {
                formValues.reset()
                navigate(`/events/e/${event.id}/lists/overview/${list.id}/settings`)
            })
        }
    })

    const newListNameRef = useRef<HTMLInputElement>(null)

    return (
        <div className={"stack"}>
            <form
                className={"section stack"}
                onSubmit={formValues.onSubmit(() => createListMutation.mutate())}
            >
                <Title order={3} c={"blue"}>
                    <Group>
                        <ActionIcon
                            mb={"5"} variant={"transparent"}
                            onClick={() => newListNameRef.current?.focus()}
                        >
                            <IconPlus/>
                        </ActionIcon>
                        Neue Liste Erstellen
                    </Group>
                </Title>

                <PocketBaseErrorAlert error={createListMutation.error}/>

                <TextInput
                    ref={newListNameRef}
                    variant={"filled"}
                    placeholder={"Name der neuen Liste ..."}
                    {...formValues.getInputProps("name")}
                />

                <Collapse in={!!formValues.values.name}>
                    <Title order={4} c={"blue"}>Beschreibung</Title>

                    <TextEditor
                        mt={"sm"}
                        value={formValues.values.description}
                        onChange={(value) => formValues.setFieldValue("description", value)}
                    />

                    <Box mt={"sm"}>
                        <Button
                            type={"submit"}
                            loading={createListMutation.isPending}
                            disabled={!formValues.isTouched()}
                        >
                            Liste Erstellen
                        </Button>
                    </Box>
                </Collapse>
            </form>

            <Autocomplete
                placeholder={"Nach einer Liste suchen ..."}
                leftSection={<IconListSearch/>}
                rightSection={listsQuery.isLoading ? <Loader size={"xs"}/> : undefined}
                data={
                    listsQuery
                        .data
                        ?.items
                        .map(e => e.name)
                        .filter(onlyUnique) ?? []
                }
                defaultValue={listSearchQuery} onChange={setListSearchQuery}
            />

            {
                // Show error if lists could not be loaded
                <PocketBaseErrorAlert error={listsQuery.error}/>
            }

            {
                listsQuery.data?.items.length === 0 ? <Alert color={"yellow"} title={"Keine Listen"}>
                        {
                            // case no lists found
                            listSearchQuery ? `Keine Listen mit dem Namen "${listSearchQuery}" gefunden.` :
                                `Für dieses Event sind keine Listen vorhanden.`
                        }
                    </Alert>
                    : <>
                        {
                            // case lists found
                            listsQuery.data?.items.map((list, index) => {
                                return (
                                    <ListRow list={list} event={event} key={index}/>
                                )
                            })
                        }
                        <Center>
                            <Pagination
                                size={"sm"}
                                value={activePage} onChange={setPage}
                                total={listsQuery.data?.totalPages ?? 0}
                            />
                        </Center>
                    </>
            }
        </div>
    )
}