import {EventListSlotEntriesWithUserModel} from "@/models/EventTypes.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {useMutation, useQuery} from "@tanstack/react-query";
import {showSuccessNotification} from "@/components/util.tsx";
import {Alert, Button, Group, Modal, Select} from "@mantine/core";
import {useEffect, useState} from "react";
import {pprintDateTime} from "@/lib/datetime.ts";
import {getUserName, RenderUserName} from "@/components/auth/modals/util.tsx";

export const MoveEventListSlotEntryModal = ({opened, close, refetch, entry}: {
    opened: boolean,
    close: () => void,
    refetch: () => void,
    entry: EventListSlotEntriesWithUserModel
}) => {

    const {pb} = usePB()

    const [selectedSlot, setSelectedSlot] = useState<string | null>(entry.eventListsSlot)

    const [selectedList, setSelectedList] = useState<string | null>(entry.eventList)

    useEffect(() => {
        setSelectedSlot(null)
    }, [selectedList]);

    const mutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("eventListSlotEntries").update(entry.id, {
                eventListsSlot: selectedSlot
            })
        },
        onSuccess: () => {
            showSuccessNotification(`Eintrag von ${getUserName(entry.expand?.user)} erfolgreich verschoben`)
            slotsQuery.refetch()
            listsQuery.refetch()
            refetch()
            close()
        }
    })

    const listsQuery = useQuery({
        queryKey: ["eventLists"],
        queryFn: async () => (
            await pb.collection("eventLists").getFullList({
                // controlled search for lists, currently not working https://github.com/react-hook-form/documentation/issues/916
                filter: `event='${entry.event}'`,
                sort: "name"
            })
        ),
        enabled: opened
    })

    const slotsQuery = useQuery({
        queryKey: ["eventListSlots", entry.eventList, "notFull", selectedList],
        queryFn: async () => (
            await pb.collection("eventListSlotsWithEntriesCount").getFullList({
                filter: `eventList='${selectedList}'
                &&(maxEntries=0 || entriesCount < maxEntries)`,
                sort: "startDate"
            })
        ),
        enabled: selectedList !== null && opened
    })

    return <Modal
        size={"lg"}
        opened={opened}
        onClose={close}
        title={<> Eintrag von <b><RenderUserName user={entry.expand?.user}/></b> verschieben </>}
    >
        <form
            onSubmit={(e) => {
                e.preventDefault()
                mutation.mutate()
            }}
            className={"stack"}
        >
            <PocketBaseErrorAlert error={mutation.error}/>

            <Select
                label={"Liste"}
                description={"Wähle die neue Liste für den Eintrag aus"}
                placeholder={"Liste auswählen"}
                allowDeselect={false}

                searchable
                nothingFoundMessage="Keine Listen gefunden..."

                data={listsQuery.data?.map(list => ({
                    value: list.id,
                    label: list.name
                })) ?? []}
                value={selectedList}
                onChange={(value) => setSelectedList(value)}
            />

            <Select
                label={"Zeitslot"}
                description={"Wähle den neuen Zeitslot für den Eintrag aus"}
                placeholder={"Zeitslot auswählen"}
                data={slotsQuery.data?.map(slot => ({
                    value: slot.id,
                    label: `${pprintDateTime(slot.startDate)} - ${pprintDateTime(slot.endDate)}`
                })) ?? []}
                value={selectedSlot}
                onChange={(value) => setSelectedSlot(value)}
            />

            {
                selectedList !== entry.eventList && (
                    <Alert color={"orange"} title={"Achtung!"}>
                        Wenn ein Eintrag in eine andere Liste mit anderen Formular- und Statusfeldern verschoben wird,
                        gehen keine Daten verloren, werden aber nicht mehr angezeigt.
                    </Alert>
                )
            }

            <Group>
                <Button onClick={close} variant={"light"}>Abbrechen</Button>
                <Button type={"submit"} disabled={!selectedSlot}>Speichern</Button>
            </Group>
        </form>
    </Modal>
}