import {EventListModel, EventListSlotModel} from "@/models/EventTypes.ts";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {useForm} from "@mantine/form";
import {useMutation} from "@tanstack/react-query";
import {useConfirmModal} from "@/components/ConfirmModal.tsx";
import {Button, Center, Group, NumberInput} from "@mantine/core";
import {DateTimePicker} from "@mantine/dates";
import TextEditor from "@/components/input/Editor";
import ShowHelp from "@/components/ShowHelp.tsx";
import {formatDuration} from "@/lib/datetime.ts";

export default function UpsertEventListSlot({list, slot, onSuccess, onAbort}: {
    list: EventListModel,
    slot?: EventListSlotModel,
    onSuccess: () => void,
    onAbort?: () => void
}) {

    const {pb} = usePB()

    const formValues = useForm({
        initialValues: {
            startDate: slot?.startDate ? new Date(slot.startDate) : new Date(),
            endDate: slot?.endDate ? new Date(slot.endDate) : new Date(),
            maxEntries: slot?.maxEntries ?? null,
            description: slot?.description ?? ""
        },
        validate: {
            startDate: (val) => {
                if (!val) {
                    return "Startdatum muss angegeben werden"
                }
            },
            endDate: (val, values) => {
                if (!val) return "Enddatum muss angegeben werden"
                if (val < values.startDate) return "Enddatum muss nach dem Startdatum liegen"
                return null
            },
            maxEntries: (val) => val && val < 0 ? "Maximale Anzahl an Plätzen muss größer oder gleich 0 sein" : null
        }
    })

    const upsertSlotMutation = useMutation({
        mutationFn: async () => {
            if (slot) {
                return await pb.collection("eventListSlots").update(slot.id, {
                    ...formValues.values,
                    eventList: list.id
                })
            } else {
                return await pb.collection("eventListSlots").create({
                    ...formValues.values,
                    eventList: list.id
                })
            }
        },
        onSuccess: onSuccess
    })

    const deleteSlotMutation = useMutation({
        mutationFn: async () => {
            return await pb.collection("eventListSlots").delete(slot!.id)
        },
        onSuccess: onSuccess
    })

    const {toggleConfirmModal, ConfirmModal} = useConfirmModal({
        description: "Möchtest du diesen Slot wirklich löschen? Damit werden auch alle Anmeldungen für diesen Slot gelöscht.",
        onConfirm: deleteSlotMutation.mutate,
    })

    const duration = formatDuration(formValues.values.startDate, formValues.values.endDate)

    return <form className={"stack"} onSubmit={formValues.onSubmit(() => upsertSlotMutation.mutate())}>
        <ConfirmModal/>

        <PocketBaseErrorAlert error={upsertSlotMutation.error}/>

        <TextEditor
            placeholder={"Beschreibung ..."}
            value={formValues.values.description}
            onChange={(val) => formValues.setFieldValue("description", val)}
        />

        <ShowHelp>
            Wenn der Zeitslot von einem Tag um 00:00 Uhr zu einem anderen Tag um 00:00 Uhr geht,
            wird der Zeitslot als ganztägig angezeigt.
            <br/>
            <br/>
            Der Zeitslot dauert insgesamt <b>{duration}</b>
        </ShowHelp>

        <Group justify={"center"} grow>
            <DateTimePicker
                size="xs"
                label={"Startdatum & Zeit"}
                required
                {...formValues.getInputProps("startDate")}
            />

            <DateTimePicker
                size="xs"
                label={"Enddatum & Zeit"}
                required
                {...formValues.getInputProps("endDate")}
            />

            <NumberInput
                size="xs"
                label={"Maximale Anzahl an Plätzen"}
                placeholder={"Leer lassen für unbegrenzte Plätze"}
                {...formValues.getInputProps("maxEntries")}
            />
        </Group>

        <Center>
            <Button.Group>
                {
                    onAbort && <Button
                        variant={"light"}
                        size={"xs"}
                        aria-label={"Abort"}
                        onClick={onAbort}
                        color={"gray"}
                    >
                        Abbrechen
                    </Button>
                }

                {
                    slot && <Button
                        variant={"light"}
                        size={"xs"}
                        aria-label={"Delete"}
                        color={"red"}
                        onClick={() => toggleConfirmModal()}
                    >
                        Löschen
                    </Button>
                }

                <Button
                    variant={"light"}
                    size={"xs"}
                    type={"submit"}
                    loading={upsertSlotMutation.isPending}
                    color={"teal"}
                    aria-label={"Save"}
                >
                    Speichern
                </Button>
            </Button.Group>
        </Center>
    </form>
}