import PromptLoginModal from "@/components/auth/modals/PromptLoginModal.tsx";
import {Link, useNavigate} from "react-router-dom";
import {useState} from "react";
import {useDebouncedValue, useToggle} from "@mantine/hooks";
import {useQuery} from "@tanstack/react-query";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {
    ActionIcon,
    Anchor,
    Breadcrumbs,
    Center,
    Group,
    Loader,
    LoadingOverlay,
    Pagination,
    Stack,
    Text,
    TextInput,
    ThemeIcon,
    Title,
    Tooltip
} from "@mantine/core";
import ShowHelp from "@/components/ShowHelp.tsx";
import {IconConfetti, IconDatabaseOff, IconSortAscending, IconSortDescending} from "@tabler/icons-react";
import UserEntryRow from "@/pages/events/entries/UserEntryRow.tsx";

export default function UserEntries() {

    const navigate = useNavigate()

    const {pb, user} = usePB()

    const [page, setPage] = useState(1)

    const [sortDirection, toggleSortDirection] = useToggle(['DESC', 'ASC']);

    const [eventSearch, setEventSearch] = useState("")

    const [debouncedSearch] = useDebouncedValue(eventSearch, 200)

    const entriesQuery = useQuery({
        queryKey: ["userEntries", user?.id, page, debouncedSearch, sortDirection],
        queryFn: async () => (
            await pb.collection("eventListSlotEntriesWithUser").getList(page, 50, {
                filter: `user='${user?.id}'${debouncedSearch ? `&&event.name~'${debouncedSearch}'` : ""}`,
                sort: sortDirection === "ASC" ? "-event.startDate" : "event.startDate",
                expand: "user"
            })
        ),
        enabled: !!user
    })

    if (!user) {
        return <PromptLoginModal onAbort={() => navigate("/")}/>
    }

    return <>
        {entriesQuery.isLoading && <LoadingOverlay/>}

        <div className={"section-transparent"}>
            <Breadcrumbs>{[
                {title: "Home", to: "/"},
                {title: "Events", to: "/events"},
                {title: "Anmeldungen", to: `/events/entries`},
            ].map(({title, to}) => (
                <Anchor component={Link} to={to} key={title}>
                    {title}
                </Anchor>
            ))}</Breadcrumbs>
        </div>
        <div className={"section"}>
            <Title order={1} c={"blue"}>Anmeldungen</Title>
        </div>
        <div className={"section-transparent"}>
            <ShowHelp>
                Auf dieser Seite findest du alle deine Anmeldungen zu Events und kannst diese verwalten.
            </ShowHelp>
        </div>

        <div className={"section stack"}>
            <TextInput
                value={eventSearch}
                onChange={ev => setEventSearch(ev.target.value)}
                leftSection={<IconConfetti/>}
                rightSection={entriesQuery.isLoading ? <Loader size={"xs"}/> : (
                    <Tooltip label={`${
                        sortDirection === "ASC" ? "Events (alt → neu)" : "Events (neu → alt)"
                    }`} withArrow>
                        <ActionIcon
                            onClick={() => toggleSortDirection()}
                            variant={"transparent"}
                            aria-label={"toggle sort"}
                        >
                            {sortDirection === "ASC" ? <IconSortAscending/> : <IconSortDescending/>}
                        </ActionIcon>
                    </Tooltip>
                )}
                placeholder={"Nach Events suchen ..."}
            />

            <Group justify={"space-between"}>
                <Text c={"dimmed"} size={"xs"}>
                    {eventSearch ? `Suche nach Anmeldungen für Event '${eventSearch}'` : "Alle deine Anmeldungen"}
                </Text>

                <Text c={"dimmed"} size={"xs"}>
                    {entriesQuery.data?.totalItems ?? 0} {eventSearch ? "Ergebnisse" : "Anmeldungen"}
                </Text>
            </Group>

            <PocketBaseErrorAlert error={entriesQuery.error}/>

            {entriesQuery.data?.totalItems === 0 && <Stack align={"center"}>
                <ThemeIcon variant={"transparent"} color={"gray"} size={"md"}>
                    <IconDatabaseOff/>
                </ThemeIcon>

                <Title order={4} c={"dimmed"}>Keine Anmeldungen vorhanden</Title>
            </Stack>
            }

            {entriesQuery.data?.items.map(entry => (
                <UserEntryRow entry={entry} refetch={() => entriesQuery.refetch()} key={entry.id}/>
            ))}

            <Center>
                <Pagination total={entriesQuery.data?.totalPages ?? 1} value={page} onChange={setPage} size={"xs"}/>
            </Center>

        </div>
    </>
}