import {EventListSlotEntriesWithUserModel} from "@/models/EventTypes.ts";
import {ActionIcon, Collapse, Group, Text, ThemeIcon, Tooltip} from "@mantine/core";
import {IconChevronDown, IconChevronRight, IconConfetti, IconForms, IconList, IconTrash} from "@tabler/icons-react";
import TextWithIcon from "@/components/layout/TextWithIcon";
import {
    EventListSlotEntryDetails
} from "@/pages/events/e/:eventId/EventLists/:listId/EventListSlotsTable/EventListSlotEntryRow.tsx";

import {useDisclosure} from "@mantine/hooks";
import {RenderDateRange} from "@/pages/events/e/:eventId/EventLists/components/RenderDateRange.tsx";
import classes from "./UserEntryRow.module.css"
import {humanDeltaFromNow} from "@/lib/datetime.ts";
import {useMutation} from "@tanstack/react-query";
import {showSuccessNotification} from "@/components/util.tsx";
import {useConfirmModal} from "@/components/ConfirmModal.tsx";
import {usePB} from "@/lib/pocketbase.tsx";
import {
    UpdateEventListSlotEntryFormModal
} from "@/pages/events/e/:eventId/EventLists/components/UpdateEventListSlotEntryFormModal.tsx";
import {getUserName} from "@/components/auth/modals/util.tsx";

export default function UserEntryRow({entry, refetch}: {
    entry: EventListSlotEntriesWithUserModel,
    refetch: () => void
}) {
    const {pb} = usePB()

    const [expanded, expandedHandler] = useDisclosure(false)

    const [showEditFormModal, showEditFormModalHandler] = useDisclosure(false)

    const delta = humanDeltaFromNow(entry.slotStartDate, entry.slotEndDate)

    const deleteEntryMutation = useMutation({
        mutationFn: async () => {
            await pb.collection("eventListSlotEntries").delete(entry.id)
        },
        onSuccess: () => {
            refetch()
            showSuccessNotification("Eintrag gelöscht")
        }
    })

    const {ConfirmModal, toggleConfirmModal} = useConfirmModal({
        title: 'Eintrag löschen',
        description: `Möchtest du den Eintrag von ${getUserName(entry.expand?.user)} wirklich löschen?`,
        onConfirm: () => deleteEntryMutation.mutate()
    })

    return <div className={classes.container}>

        <ConfirmModal/>

        <UpdateEventListSlotEntryFormModal
            opened={showEditFormModal}
            close={showEditFormModalHandler.close}
            refetch={refetch}
            entry={entry}
        />

        <div className={classes.row}>
            <Tooltip label={expanded ? "Details verbergen" : "Details anzeigen"} withArrow>
                <ActionIcon onClick={expandedHandler.toggle} variant={"light"} size={"xs"}>
                    {expanded ? <IconChevronDown/> : <IconChevronRight/>}
                </ActionIcon>
            </Tooltip>


            <TextWithIcon icon={
                <ThemeIcon variant={"transparent"} size={"xs"} color={"gray"}>
                    <IconConfetti/>
                </ThemeIcon>
            }>
                {entry.eventName}
            </TextWithIcon>

            <TextWithIcon icon={
                <ThemeIcon variant={"transparent"} size={"xs"} color={"gray"}>
                    <IconList/>
                </ThemeIcon>
            }>
                {entry.listName}
            </TextWithIcon>

            <Group gap={"xs"}>
                <RenderDateRange start={new Date(entry.slotStartDate)} end={new Date(entry.slotEndDate)}/>
                <Text size={"sm"} c={"dimmed"}>
                    {delta.message}
                </Text>
            </Group>

            <Group gap={"xs"}>
                <Tooltip
                    label={delta.delta === "PAST" ? "Vergangene Anmeldungen können nicht bearbeiten werden" : "Eintrag bearbeiten"}
                    withArrow>
                    <ActionIcon
                        variant={"light"}
                        size={"xs"}
                        onClick={showEditFormModalHandler.toggle}
                        aria-label={"edit entry form"}
                        disabled={delta.delta === "PAST"}
                    >
                        <IconForms/>
                    </ActionIcon>
                </Tooltip>

                <Tooltip
                    label={delta.delta === "PAST" ? "Vergangene Anmeldungen können nicht gelöscht werden" : "Eintrag löschen"}
                    withArrow>
                    <ActionIcon
                        variant={"light"}
                        color={"red"}
                        size={"xs"}
                        onClick={toggleConfirmModal}
                        aria-label={"delete entry"}
                        disabled={delta.delta === "PAST"}
                    >
                        <IconTrash/>
                    </ActionIcon>
                </Tooltip>
            </Group>
        </div>

        <Collapse in={expanded}>
            <EventListSlotEntryDetails entry={entry}/>
        </Collapse>
    </div>
}