import {EventListModel, EventListSlotsWithEntriesCountModel} from "@/models/EventTypes.ts";
import classes from "./EventListSlotView.module.css";
import {RenderDateRange} from "@/pages/events/e/:eventId/EventLists/components/RenderDateRange.tsx";
import EventListSlotProgress from "@/pages/events/e/:eventId/EventLists/components/EventListSlotProgress.tsx";
import {useDisclosure} from "@mantine/hooks";
import {Alert, Collapse, ThemeIcon, Tooltip, UnstyledButton} from "@mantine/core";
import {IconChevronDown, IconChevronRight, IconInfoCircle} from "@tabler/icons-react";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {useMutation} from "@tanstack/react-query";
import {FieldEntries} from "@/components/formUtil/FromInput/types.ts";
import {showSuccessNotification} from "@/components/util.tsx";
import InnerHtml from "@/components/InnerHtml";
import FormInput from "@/components/formUtil/FromInput";
import {useNavigate} from "react-router-dom";

export default function EventListSlotView({slot, list, refetch}: {
    list: EventListModel,
    slot: EventListSlotsWithEntriesCountModel,
    refetch: () => void
}) {

    const [expanded, expandedHandler] = useDisclosure(false)

    const slotIsFull = slot.maxEntries === 0 || slot.maxEntries === null ? false : slot.entriesCount >= slot.maxEntries

    const slotIsInPast = new Date(slot.endDate) < new Date()

    const schema = {
        fields: [
            ...slot.entryQuestionSchema?.fields ?? [],
            ...slot.defaultEntryQuestionSchema?.fields ?? [],
        ]
    }

    const {pb, user} = usePB()

    const navigate = useNavigate()

    const createEntryMutation = useMutation({
        mutationFn: async (data: FieldEntries) => {
            await pb.collection("eventListSlotEntries").create({
                entryQuestionData: data,
                eventListsSlot: slot.id,
                user: user?.id
            })
        },
        onSuccess: () => {
            showSuccessNotification("Eintrag erfolgreich erstellt")
            expandedHandler.close()
            refetch()
            navigate(`/events/entries`)
        }
    })

    return (<>
            <UnstyledButton onClick={expandedHandler.toggle} className={classes.row}>
                <Tooltip label={expanded ? "Ausblenden" : "Eintragen"} withArrow>
                    <ThemeIcon variant={"light"} size={"xs"}>
                        {expanded ? <IconChevronDown/> : <IconChevronRight/>}
                    </ThemeIcon>
                </Tooltip>

                <RenderDateRange start={new Date(slot.startDate)} end={new Date(slot.endDate)}/>

                <EventListSlotProgress slot={slot}/>
            </UnstyledButton>

            <Collapse in={expanded} className={"stack"}>
                <PocketBaseErrorAlert error={createEntryMutation.error}/>

                {
                    slot.description &&
                    <Alert icon={<IconInfoCircle/>}>
                        <InnerHtml html={slot.description}/>
                    </Alert>
                }

                {

                    list.onlyStuVeAccounts && user?.REALM !== "LDAP" ? <>
                        <Alert color={"red"}>
                            Für diese Liste sind nur StuVe-Accounts zugelassen
                        </Alert>
                    </> : slotIsInPast ? <>
                        <Alert color={"red"}>
                            Dieser Zeitslot ist bereits vorbei
                        </Alert>
                    </> : slotIsFull ? <>
                            <Alert color={"red"} title={"Zeitslot voll"}>
                                Dieser Zeitslot ist bereits voll
                            </Alert>
                        </>
                        :
                        <FormInput
                            disabled={!user || slotIsFull}
                            schema={schema}
                            onSubmit={createEntryMutation.mutateAsync}
                        />
                }
            </Collapse>
        </>
    )
}