import {EventModel} from "@/models/EventTypes.ts";
import {useQuery} from "@tanstack/react-query";
import {PocketBaseErrorAlert, usePB} from "@/lib/pocketbase.tsx";
import {Alert, Center, Loader, Pagination, Title} from "@mantine/core";
import InnerHtml from "@/components/InnerHtml";
import {IconInfoCircle, IconLock} from "@tabler/icons-react";
import {useState} from "react";
import EventListSlotView from "@/pages/events/s/EventListSlotView.tsx";

export default function EventListView({event, listId}: { event: EventModel, listId: string }) {

    const {pb} = usePB()

    const [activePage, setPage] = useState(1)

    const listQuery = useQuery({
        queryKey: ["event", "list", listId],
        queryFn: async () => (await pb.collection("eventLists").getOne(listId, {
            requestKey: listId
        })),
        enabled: !!listId
    })

    const listSlotsQuery = useQuery({
        queryKey: ["event", event.id, "list", listId, "slots", activePage],
        queryFn: async () => (await pb.collection("eventListSlotsWithEntriesCount").getList(activePage, 10, {
            filter: `eventList='${listId}'`,
            sort: "startDate",
            enabled: !!listId,
            requestKey: listId
        }))
    })

    if (listSlotsQuery.isLoading || listQuery.isLoading) {
        return <div className={"section center"}>
            <Loader size={"sm"}/>
        </div>
    }

    if (listSlotsQuery.error || listQuery.error || !listQuery.data) {
        return <div className={"section"}>
            <PocketBaseErrorAlert error={listSlotsQuery.error ?? listQuery.error}/>
        </div>
    }

    const slots = listSlotsQuery.data?.items ?? []
    const list = listQuery.data

    return <div className={"section stack"}>
        <Title order={3}>{list.name}</Title>

        {!list.open && (
            <Alert icon={<IconLock/>}>
                Diese Liste ist nicht für Anmeldungen geöffnet
            </Alert>
        )}

        {
            list.description &&
            <Alert icon={<IconInfoCircle/>}>
                <InnerHtml html={list.description}/>
            </Alert>
        }

        {
            !list.allowOverlappingEntries &&
            <Alert>
                Wenn du bei diesem Event bereits für einen anderen Zeitslot angemeldet bist,
                kannst du in dieser Liste nur einen Zeitslot wählen,
                der nicht mit deinen anderen Anmeldungen kollidiert.
            </Alert>
        }

        {
            list.onlyStuVeAccounts &&
            <Alert>
                Für diese Liste können sich nur Personen mit StuVe Account anmelden
            </Alert>
        }


        {
            (list.open && slots.length !== 0) && <>
                {slots.map(slot => (
                    <EventListSlotView key={slot.id} list={list} slot={slot} refetch={() => {
                        listSlotsQuery.refetch()
                    }}/>
                ))}
                {(listSlotsQuery.data?.totalPages ?? 0) > 1 &&
                    <Center>
                        <Pagination
                            size={"xs"}
                            value={activePage}
                            onChange={setPage}
                            total={listSlotsQuery.data?.totalPages ?? 0}
                        />
                    </Center>
                }
            </>
        }
    </div>
}