import {Link, useParams, useSearchParams} from "react-router-dom";
import {usePB} from "@/lib/pocketbase.tsx";
import {useQuery} from "@tanstack/react-query";
import NotFound from "../../not-found/index.page.tsx";
import {Accordion, Alert, Anchor, Breadcrumbs, Button, Center, Group, Loader, Title} from "@mantine/core";
import PBAvatar from "@/components/PBAvatar.tsx";
import InnerHtml from "@/components/InnerHtml";
import {IconExternalLink, IconLogin, IconPencil, IconSectionSign} from "@tabler/icons-react";
import EventData from "@/pages/events/e/:eventId/EventComponents/EventData.tsx";
import EventListView from "@/pages/events/s/EventListView.tsx";
import {useEventRights} from "@/pages/events/util.ts";


export default function SharedEvent() {

    const {pb,user} = usePB()


    const {eventId} = useParams() as { eventId: string }

    const [searchParams] = useSearchParams()
    const listIds = searchParams.get('lists')?.split(",") ?? []


    const eventQuery = useQuery({
        queryKey: ["event", eventId],
        queryFn: async () => (await pb.collection("events").getOne(eventId)),
        enabled: !!eventId
    })

    const {canEditEventList, canEditEvent} = useEventRights(eventQuery.data)

    if (eventQuery.isLoading) {
        return <Center h={"100%"}><Loader/></Center>
    }

    if (!eventQuery.data || eventQuery.isError) {
        return <NotFound/>
    }

    const event = eventQuery.data

    return <>
        <div className={"section-transparent stack center"}>
            <PBAvatar
                size={200} radius={1000} model={event}
                name={event.name} img={event.img}
            />
            <Title order={1} style={{overflowWrap: "break-word", width: "calc(100%)"}} ta={"center"}>
                {event.name}
            </Title>
        </div>

        {!user && <div className={"section-transparent"}>
            <Alert icon={<IconLogin/>} color={"orange"}>
                Um dich in eine Liste einzutragen, musst du dich anmelden
            </Alert>
        </div>}

        <div className={"section-transparent"}>
            <Breadcrumbs>{[
                {title: "Home", to: "/"},
                {title: "Events", to: "/events"},
                {title: event.name, to: `/events/${event.id}`},
            ].map(({title, to}) => (
                <Anchor component={Link} to={to} key={title}>
                    {title}
                </Anchor>
            ))}</Breadcrumbs>
        </div>

        {(canEditEvent || canEditEventList) && <div className={"section-transparent"}>
            <Alert color={"green"} title={"Du kannst dieses Event bearbeiten"}>
                <Button
                    component={"a"}
                    href={`/events/${eventId}`}
                    variant={"light"}
                    leftSection={<IconPencil/>}
                >
                    Event bearbeiten
                </Button>
            </Alert>
        </div>}

        <EventData hideHeader event={event}/>

        {event.eventLinks !== null && event.eventLinks.length !== 0 && <div className={"section-transparent"}>
            <Group>
                {event.eventLinks.map((link, index) => (
                    <Button
                        key={index}
                        component={"a"}
                        href={link.url}
                        target={"_blank"}
                        leftSection={<IconExternalLink/>}
                    >
                        {link.name}
                    </Button>
                ))}
            </Group>
        </div>}

        {event.description && (
            <div className={"section"}>
                <InnerHtml html={event.description}/>
            </div>
        )}

        {
            (event.isStuveEvent || event.additionalAgb) && (
                <div className={"section stack"}>
                    <Accordion variant="separated">
                        {event.isStuveEvent && (
                            <Accordion.Item value={"stuve-agb"}>
                                <Accordion.Control icon={<IconSectionSign/>}>AGB der Stuve</Accordion.Control>
                                <Accordion.Panel>
                                    <Alert title={"StuVe Event"}>
                                        Dieses Event wird im Rahmen der StuVe durchgeführt. Die AGB der StuVe sind für
                                        dieses Event gültig.
                                        <br/>

                                        <Button
                                            mt={"xs"}
                                            component={Link}
                                            to={`/legal/terms-and-conditions`}
                                            variant={"light"}
                                            target={"_blank"}
                                            leftSection={<IconExternalLink/>}
                                        >
                                            AGB der StuVe
                                        </Button>
                                    </Alert>

                                </Accordion.Panel>
                            </Accordion.Item>
                        )}
                        {event.additionalAgb && (
                            <Accordion.Item value={"event-agb"}>
                                <Accordion.Control icon={<IconSectionSign/>}>AGB des Events</Accordion.Control>
                                <Accordion.Panel><InnerHtml html={event.additionalAgb}/></Accordion.Panel>
                            </Accordion.Item>
                        )}
                    </Accordion>
                </div>
            )
        }

        {
            listIds.map(listId => (
                <EventListView event={event} listId={listId} key={listId}/>
            ))
        }
    </>
}